<?php
include_once 'webdav.php';

//************************************************************
// Hlavn tda realizujc CalDAV protokol
// Main class implementing CalDAV protocol
//************************************************************

class caldav_session extends webdav_session
{
  var $_version;
  
  function caldav_session()
  {
    $this->webdav_session();
  }

//************************************************************

  function if_match()
  {
    //Vracme edit counter
    //We return edit counter
    if ($if_match=$_SERVER['HTTP_IF_MATCH'])
    {
      $if_match=trim($if_match,'"');
      $p=strpos($if_match,':');
      $if_match=substr($if_match,$p+1,strlen($if_match)-$p-1);
    }
    return $if_match;
  }

//************************************************************
  
  function handle_delete()
  { 
    //Aktualizujeme stvajc poloku?
    //Do we update existing item?    
    $if_match=$this->if_match();
    
    $status=$this->_datastore->delete_item($this->_folder,$this->_local_id,$if_match);
    
    $this->http_response($status,$headers);          
  }

//************************************************************
  
  function handle_move()
  {    
    $destination=$_SERVER['HTTP_DESTINATION'];
    $status=$this->_datastore->move_item($this->_folder,$this->_local_id,$destination);
    
    $this->http_response($status);          
  }

//************************************************************
  
  function handle_mkcalendar()
  {    
    $status=$this->_datastore->mkcalendar($this->_folder, $this->_body);
    
    $this->http_response($status);          
  }

//************************************************************
  
  function handle_mkcol()
  {    
    return $this->handle_mkcalendar();
  }

//************************************************************
  
  function handle_get()
  {
    $result=$this->_datastore->get_item($this->_folder,$this->_local_id);
    
    //Existuje poloka, kterou chceme vrtit?
    //Does item we want to get exist?
    if ($result['item'])
      $headers=array('Content-Type: '.$result['content_type'],'ETag: '.$this->_local_id.':'.$result['edit_counter']);
    else
      $headers=array();
    
    $this->http_response($result['status'],$headers,$result['item']);  
  }

//************************************************************
  
  function handle_head() 
  {
    $this->handle_get();
  }

//************************************************************
  
  function handle_put()
  {            
    //Vytvme novou poloku?
    //Do we create new item?
    if ($_SERVER['HTTP_IF_NONE_MATCH']=='*')
      $verify=';VERIFY';
        
    //Aktualizujeme stvajc poloku?
    //Do we update existing item?
    $if_match=$this->if_match();  
    
    $result=$this->_datastore->put_item($this->_folder,$this->_local_id,$this->_body,$verify,$if_match);  
          
    $this->http_response($result['status']);
  }

//************************************************************
  
  function handle_report()
  {    
    $request=text_to_xml($this->_body);
    $report_type=$request->get_name();
    
    switch($report_type)
    {
      case 'calendar-query':
      case 'propfind':
        $result=$this->item_query_report($request);
        break;
      case 'calendar-multiget':
        $result=$this->calendar_multiget_report($request);
        break;
      default:
        $this->http_response('405 Method not allowed');
        return;  
    }    
    $this->caldav_multistatus($result);   
  }

//************************************************************
  
  function item_query_report($request)
  {
    //Chce klient vrtit kolekce?
    //Does client want to collections?
    $resourcetype=&$request->get_child('prop/resourcetype');
    $this->_datastore->set_folder($resourcetype?true:false);

    //Chce klient vrtit data?
    //Does client want to send data?
    $calendar_data=&$request->get_child('prop/calendar-data');
    $this->_datastore->set_data($calendar_data?true:false);
    
    //Chce klient vrtit etag?
    //Does client want to send etag?
    $etag=&$request->get_child('prop/getetag');
    $this->_datastore->set_etag($etag?true:false);
    
    //Chce klient vrtit opakujc se data vicekrt?
    //Does client want to expand repeating data? 
    if ($calendar_data)  
      $expand=&$calendar_data->get_child('expand');
    $this->_datastore->set_expand($expand?true:false);      
    
    //Chce klient vrtit konkrtn typ dat ci data z njakho asovho intervalu?
    //Does client want to send concreete type objects or data from any time range?
    $filter=&$request->get_child('filter');
    if ($filter)
    {
      $comp_filter=&$filter->get_child('comp-filter/comp-filter');
      $time_range=&$filter->get_child('comp-filter/comp-filter/time-range');  
    }
    
    $this->_datastore->set_object_type($comp_filter->_attribs);
    $this->_datastore->set_julian_range($time_range->_attribs);

    if (!isset($this->_depth))
      $this->_depth='infinity';
    $this->_datastore->set_depth($this->_depth);

    $result=array();  
    $this->_datastore->get_items($this->_folder,$this->_local_id,&$result);   
    return $result;
  }

//************************************************************

  function calendar_multiget_report($request)
  {
    //UNDER CONSTRUCTION
  }

//************************************************************
  
  function caldav_multistatus($results)
  {
    global $collection_types;
      
    $multistatus=new xml_tree_node('D:multistatus',array('xmlns:D'=>'DAV:','xmlns:C'=>'urn:ietf:params:xml:ns:caldav'));
    foreach($results as $id=>$result)
    {
      $response=&$multistatus->new_child('D:response',NULL);
      
      // Return items
      if (isset($result['counter'])) 
      {
        $response->new_child_data('D:href',NULL,$this->_server.'/'.str_replace('\\','/',$result['folder']).'/'.$id);
        
        //Existuje poloka, kterou chceme vrtit?
        //Does item we want to get exist?
        if ($result['data'])
        {
          $propstat=&$response->new_child('D:propstat',NULL);
          $prop=&$propstat->new_child('D:prop',NULL);
          $prop->new_child_data('D:getetag',NULL,$id.':'.$result['counter']);
          
          //Mme poloku, jej obsah chceme vrtit?
          //Have we item we wants to get its content?
          if ($result['data']!='*')
            $prop->new_child_data('C:calendar-data',NULL,$result['data']);
          
          $propstat->new_child_data('D:status',NULL,'HTTP/1.1 200 OK');
        }
        else
          $response->new_child_data('D:status',NULL,'HTTP/1.1 404 Resource not found');                      
      }
      
      // Return collection list
      else
      {
        $response->new_child_data('D:href',NULL,$this->_server.'/'.$id.'/');

        $propstat=&$response->new_child('D:propstat',NULL);
        $prop=&$propstat->new_child('D:prop',NULL);

        $prop->new_child_data('D:displayname',NULL,$id);

        $resourcetype=&$prop->new_child('D:resourcetype',array('xmlns:D'=>'DAV:','xmlns:G'=>'http://groupdav.org/'));
        $resourcetype->new_child('D:collection');
//if ($result['type']!='C')
				$resourcetype->new_child('C:calendar');
        $resourcetype->new_child('G:'.$collection_types[$result['type']]);

        $propstat->new_child_data('D:status',NULL,'HTTP/1.1 200 OK');
     
      }
    }

    $response_str='<?xml version="1.0" encoding="UTF-8"?'.'>'."\r\n".$multistatus->print_node(0,false);
    $headers=array('Content-Type: '.$_SERVER['CONTENT_TYPE']);  

    $this->http_response('207 Multi-Status',$headers,$response_str);
  }

//************************************************************
  
  function handle_propfind()
  {
    $this->handle_report();
  }
}
?>
