_me = frm_add_folder.prototype;
function frm_add_folder(){};

/**
 * @brief   Form for adding folder.
 * Form has three input objects:
 *    - text input for name of the folder (folder must
 *    match this regular expr. /^ *[a-zA-Z0-9#\.\_\-]+[ a-zA-Z0-9#\.\_\-]*$/).
 *    - select box for folder type specification (mail, contact, event, journal, note, task).
 *    - folder three.
 * 'OK' button is active only if the name of the folder match the regular expr. above. 
 *     
 * @param[in]  sAccountID  [string] Only folders from this account will be listed because of
 * moving folders between accounts is disabled. 
 * @param[in]  sFolderID   [string] Optional. Forder with this ID will be preselected. 
 * 
 * @see  objects/frm_add_folder.xml
 * @see  inc/frm_add_folder.js
 */   
_me.__constructor = function(sAccountID, sFolderID){
	var me = this;

	if (!Is.Defined(sAccountID)) return;

	this._title('FORM_FOLDERS::CREATE_NEW_FOLDER');
	this._size(300,400);

	// Create formular from template
	this._draw('frm_add_folder', 'main');
	this.x_btn_ok._disabled(true);

	this.input_name._onerror = function(isError) { me.x_btn_ok._disabled(isError); }
    this.input_name._onsubmit = function(){
		if (this._value() && !count(this._checkError))
		     me.x_btn_ok._onclick();
	};
    
	if (Is.Defined(sFolderID)) {
		// Preselect combo with parent type as default
		var sDefaultType = dataSet.get('folders',[sAccountID,sFolderID,'TYPE']);
	}

	// Preselect folder
	sFullFolderPath = sAccountID + ((sFolderID) ? '/'+sFolderID : '');

	//Fill TYPE option
	var aFill,Typ;
	if (sPrimaryAccountGW>0){
		var dgw = GWOthers.getItem('RESTRICTIONS', 'disable_gw_types');

        aFill = {};
		//if (sDefaultType!='A'){}
			aFill.M = getLang('FOLDER_TYPES::MAIL');
			Typ = 'M';

		
		if(!dgw || dgw.indexOf('c')<0){
			aFill.C = getLang('FOLDER_TYPES::CONTACT');
			Typ = Typ || 'C';
		}
		if(!dgw || dgw.indexOf('e')<0){
			aFill.E = getLang('FOLDER_TYPES::EVENT');
			Typ = Typ || 'E';
		}
		if(!dgw || dgw.indexOf('j')<0){
			aFill.J = getLang('FOLDER_TYPES::JOURNAL');
			Typ = Typ || 'J';
		}
		if(!dgw || dgw.indexOf('n')<0){
			aFill.N = getLang('FOLDER_TYPES::NOTE');
			Typ = Typ || 'N';
		}
		if(!dgw || dgw.indexOf('t')<0){
			aFill.T = getLang('FOLDER_TYPES::TASK');
			Typ = Typ || 'T';
		}
		if(!dgw || dgw.indexOf('f')<0){
			aFill.F = getLang('FOLDER_TYPES::FILE');
			Typ = Typ || 'F';
		}
	}
	else{
		aFill = {M:getLang('FOLDER_TYPES::MAIL')};
		Typ = 'M';
	}

	this.select_type._fill(aFill);
    this.select_type._value("MCEJNTF".indexOf(sDefaultType)>-1?sDefaultType:Typ);

	//if folder is not listed then select account
	if (!this.tree_folder._setActive(sFullFolderPath)){
		sFullFolderPath = sAccountID;
    	this.tree_folder._setActive(sFullFolderPath)
	}
	//this.tree_folder._openup('minus',sFullFolderPath);

	// Disable/Enable 'OK' button depending on reg. expr. of folder name. Initial
	// value of obj_input is empty, so initially 'OK' button should be disabled.
	// TODO dodelat restrictions
	// this.x_btn_ok._disabled(this.input_name.__check());

	// This function is triggered when 'OK' button is pressed
	this.x_btn_ok._onclick = function() {
	
		var aPath = Path.split(me.tree_folder.__activeNode);
		var sAccountName = aPath[0];
		var sFolderName = aPath[1];

		sFolderName = ((sFolderName) ? sFolderName+'/' : '') + me.input_name._value();

		if (!dataSet.get('folders',[sAccountName,sFolderName])){
			var sFolderType = me.select_type._value();
	
			folders.add({'aid':sAccountName,'name':sFolderName,'type':sFolderType},'folders');
		}

		me._destruct();
	}
};