function wm_freebusy()
{
  this.xmlns='freebusy';
}

wm_freebusy.inherit(wm_generic);
var _me = wm_freebusy.prototype;

/**
 * @brief   Gets informations from calendar (which times an user is FREE/BUSY).
 * @param[in]  aFreebusyInfo
 *    - email  Email of the user. 
 * @param[in]  sDataSet    Dataset name.
 * @param[in]  aDataPath   Array containing path ['folder','subfolder',...] dataset.folder.subfodler.
 * @return  Array containing these items:
 *    - TYPE      BUSY | TENTATIVE | FREE
 *    - START     Starting time in unix format.
 *    - FINISH    Final time in unix format.
 */  
_me.get = function(aFreebusyInfo,sDataSet,aDataPath)
{
   var email = aFreebusyInfo['email'];

   if (!Is.Email(email))
      return false;
  
   // prepare data for request
   var aRequest = {"USER":[{"ATTRIBUTES":{"EMAIL":email}}]};

   if (!sDataSet) {
      // synchronous
      return this.parse(this.create_iq(aRequest));
   } else {
      // asynchronous
      this.create_iq(aRequest,[this,'response',[sDataSet,aDataPath]]);
      return true;
   }
}

/**
 * @brief   Callback function (from asynchronous call).
 * @param[in]  aData       Data returned from HTTP response.
 * @param[in]  sDataset    Dataset name.
 * @param[in]  aDataPath   Array containing path ['folder','subfolder',...] dataset.folder.subfodler.
 */ 
_me.response = function(aData,sDataSet,aDataPath)
{
  dataSet.add(sDataSet,aDataPath,this.parse(aData['Array']));
}

/**
 * @brief   Process data obtained from HTTP response and return them
 * in cultivated form.
 * @param[in]  aData   ['Array'] item from datas passed to callback function response()
 *    or returned from create_iq().
 * @see  response()
 * @see  create_iq()
 */ 
_me.parse = function(aData)
{
   var aItems = aData['IQ'][0]['QUERY'][0]['ITEM'];
   
   $aResult = {};
   for (var nIndex in aItems) {
      $aResult[nIndex] = {};
      $aResult[nIndex]['TYPE'] = aItems[nIndex]['TYPE'][0]['VALUE'];
      $aResult[nIndex]['START'] = aItems[nIndex]['START'][0]['VALUE'];
      $aResult[nIndex]['FINISH'] = aItems[nIndex]['FINISH'][0]['VALUE'];
   }
   
   return $aResult;
}

var freebusy = new wm_freebusy();
