<?php

/**
  @file
  @brief Script for removing garbage (old uploaded attachments etc.).
 */

require_once('inc/system.php');
require_once('inc/xml.php');

/** \brief Maximum life-time of cached files. */
define('MAX_LIFE_TIME', '30 days');

clean('./accounts');

/**
  @brief Clean garbage from all Merak accounts.
  @param $dir Merak accounts root directory.
 */
function clean($dir)
{
	if (!is_dir($dir))
		return;

	if ($dh = opendir($dir)) {
		while (($file = readdir($dh)) !== false) {
			if (($file == '.') || ($file == '..'))
				continue;

			$webmailDir = $dir . '/' . $file . '/~webmail';

			if (is_dir($webmailDir)) {
				cleanUpload($webmailDir . '/upload');
				cleanCache($webmailDir);
			}
		}
		closedir($dh);
	}
}

/**
  @brief Clean old user upload directory.
  @param $dir Upload directory.
 */
function cleanUpload($dir)
{

	if (!is_dir($dir))
		return;

	if (lifetimeExpired($dir))
		System::rmdir($dir);
}

/**
  @brief Clean old messages in an user cache.
  @param $dir Root directory of the user;
 */
function cleanCache($dir)
{
	$accounts = array();

	$doc = XML::open($dir . '/accounts.xml');
	if ($doc) {
		foreach ($doc->query('/accounts/account') as $accountNode)
			$accounts[$accountNode->getAttribute('id')] = true;
	}

	if ($dh = opendir($dir)) {
		while (($file = readdir($dh)) !== false) {
			if (($file == '.') || ($file == '..'))
				continue;

			$accountDir = $dir . '/' . $file;

			if (is_dir($accountDir)) {
				if ($file == 'upload')
					continue;

				if (!isset($accounts[$file]))
					System::rmdir($accountDir);
				else
					cleanCacheAccount($accountDir);
			}
		}
		closedir($dh);
	}
}

/**
  @brief Clean old messages in an user account cache.
  @param $dir Root directory of the account.
 */
function cleanCacheAccount($dir)
{
	$removeAccountDir = true;

	if ($dh = opendir($dir)) {
		while (($file = readdir($dh)) !== false) {
			if (($file == '.') || ($file == '..'))
				continue;

			$folderDir = $dir . '/' . $file;

			if (lifetimeExpired($folderDir))
				System::rmdir($folderDir);
			else {
				$removeAccountDir = false;

				cleanCacheFolder($folderDir);
			}
		}
		closedir($dh);
	}

	if ($removeAccountDir)
		rmdir($dir);
}

/**
  @brief Clean old messages in an user account folder cache.
  @param $dir Root directory of the folder.
 */
function cleanCacheFolder($dir)
{
	if ($dh = opendir($dir)) {
		while (($file = readdir($dh)) !== false) {
			if (($file == '.') || ($file == '..'))
				continue;

			$file = $dir . '/' . $file;

			if (lifetimeExpired($file))
				unlink($file);
		}
		closedir($dh);
	}
}

/**
  @brief Check if the life-time of a file has expired.
  @param $file Name of the file.
 */
function lifetimeExpired($file)
{
	return ((($fileTime = filemtime($file)) === false)
			|| strtotime(MAX_LIFE_TIME, $fileTime) < time());
}

?>
