<?php

/**
  @file
  @brief Script for downloading attachments.
 */
require_once('inc/get.php');
require_once('inc/user.php');
require_once('inc/mailparse.php');

function main() {

	/* Get input variables. */
	if (($sid = Get()->sid) === null)
		exit('Invalid sid');
	if (($fullpath = Get()->fullpath) === null)
		exit('Invalid fullpath');
	if (($class = Get()->class) === null)
		exit('Invalid class');

	try 
	{

		$user	= User::load($sid);
		$pathData = Tools::parseFullPath($fullpath,$class);

		switch($class)
		{
			// Send whole message / groupware item
			case 'item':
				$account_id = $pathData['account'];
				$folder_id = $pathData['folder'];
				$item_id = $pathData['item'];
				
				$account = $user->getAccount($account_id);
				$folder  = $account->getFolder($folder_id);
				$item	= $folder->getItem($item_id);
			
				$item->sendData();
			break;
			
			// Send message part / gw attachment
			case 'attachment':
				$account_id = $pathData['account'];
				$folder_id = $pathData['folder'];
				$item_id = $pathData['item'];
				$part_id = $pathData['part'];
				
				$account = $user->getAccount($account_id);
				$folder  = $account->getFolder($folder_id);
				$item	= $folder->getItem($item_id);
        $item->sendAttachment($part_id);
			break;
			
			// Export groupware
			case 'groupware':
			  //Get account to be exported
				$sAccountID = $pathData['account'];
        $oAccount = $user->getAccount($sAccountID);
        
        //Does account have an groupware account attached ?
        if(!$oGWAccount = $oAccount->gwAccount)
          throw new Exc('export_invalid_account','export_invalid_account');
        
        $sGWSessionID = $oGWAccount->gwAPI->OpenGroup('*');
        
        $sGroupWareXML = $oGWAccount->gwAPI->FunctionCall("ExportGroup",$sGWSessionID,$sAccountID);

        Tools::sendFileHeaders($sAccountID.'.xml',strlen($sGroupWareXML),'text/xml');
        echo $sGroupWareXML;
        
			break;			
			
			//TODO
			case 'allattachments':
        $account_id = $pathData['account'];
				$folder_id = $pathData['folder'];
				$item_id = $pathData['item'];

				$account = $user->getAccount($account_id);
				$folder  = $account->getFolder($folder_id);
				$item	= $folder->getItem($item_id);

        //Generate unique filename
        if ($item->wmclass=="M") 
	        $fname = $item->getSubjectFileName();
        else
          $fname = date('YmdGis');

	      $fname.='.zip';
        $filename = $_SESSION['TEMPPATH'].$fname;

	// ZIP archive
        $zip = new ZipArchive();

        //Open zip archive
        if ($zip->open($filename, ZIPARCHIVE::CREATE)!==TRUE) {
          exit("cannot open <$filename>\n");
        }
        
        //Process item attachments 
        //(GW does not check for attachments so hasAttachments property has not been set
        //so OR item class condition added)
        if ($item->hasAttachments || $item->wmclass=="GW"){
          $aAttachments = $item->getAllAttachments();
          foreach($aAttachments as $attachment){
            $content = $attachment["file_content"];
            $zip->addFromString($attachment["name"],$content);
                  
            }
        }
        
        //Clenup zip object
        $zip->close();
        
        //Send file to client
        Tools::sendFileHeaders($fname, filesize($filename), 'application/zip');
        readfile($filename);
        
        //Remove temporary file
        @unlink($filename);
			break;
			
			// Send recently uploaded attachment (stored in upload folder)
			case 'file':
				$folder_id = $pathData['folder'];
				$item_id = $pathData['item'];
	
				$userAttachments = $user->getAttachments();
				
				if (!isset($userAttachments[$folder_id][$item_id]))
					exit('Attachment doesn\'t exist');
			
				$attachment = $userAttachments[$folder_id][$item_id];
				$path = $attachment['file'];
				$size = filesize($path);
				
				$attachment['name'] = str_replace("+"," ",$attachment['name']);
				
				Tools::sendFileHeaders($attachment['name'], $size, $attachment['type']);
				readfile($path);                                
			break;
		}
	} catch (Exception $e) {
		exit($e->getMessage());
	}
}

main();

?>
