<?php

require_once('inc/cache/folder.php');
require_once('inc/imap/imap.php');

/**
  @class POP3Folder
  @brief Class representing an IMAP folder.
  @author Pavel Perna
  @date 07-08-2007
 */
class POP3Folder extends CacheFolder {

	//Type of the folder == M for mail items 
	public $type;  
	
	/**
	  @brief Create a new IMAP folder object.
	  @param $account Parent account object of the folder.
	  @param $folderID ID of the folder.
	  @param $name Name of the folder.
	  @param $rights Rights to the folder.
	 */	 
	public function __construct(&$account, $folderID, $name, $rights, $sync)
	{
    parent::__construct($account, $folderID, $name, $rights, $sync);
		$this->type = 'M';
		$this->gw = false;
		$this->itemClassName = 'POP3Item';
	
  }

	/**
	  @brief Create a new folder.
	  @param $account Parent account object of the folder.
	  @param $name Name of the folder.
	 */
	public static function create($account, $name)
	{
    throw new Exc("POP3 Folder creation is not supported","pop3_folder_create_not_supported");
	}


	/**
	  @brief Delete the folder.
	 */
	public function delete()
	{
	 throw new Exc("POP3 Folder deletion is not supported","pop3_folder_delete_not_supported");
	}
	
	/**
	  @brief Rename the folder.
	  @param $newName New name of the folder.
	 */
	public function rename($newName)
	{
	 throw new Exc("POP3 Folder rename is not supported","pop3_folder_rename_not_supported");
	}


	/**
	  @brief Delete multiple items
	 */
	public function deleteItems($oItems = false)
	{
		$account = $this->account;
		
    $pop3= POP3::instance($account);
						 
    $pop3->deleteItems($this,$oItems);
    		
		parent::deleteItems($oItems);
	}
	
	/**
	  @brief Move items from the folder to another one.
	  @param $folder Destination folder object
	 */
	public function moveItems($oFolder,$oItems = false)
	{
    throw new Exc("POP3 Item move is not supported","pop3_item_copy_not_supported");	
	}
	/**
	  @brief Copy items from the folder to another one.
	  @param $folder Destination folder object
	 */	
	public function copyItems($oFolder,$oItems = false)
	{
    throw new Exc("POP3 Item copy is not supported","pop3_item_copy_not_supported");	
	}	

	/**
	  @brief Get the list of folder items.
	  @param $filter SQL WHERE contition.
	  @param $orderBy SQL ORDER BY condition.
	  @param $offset SQL OFFSET statement.
	  @param $limit SQL LIMIT statement.
	  @return The list as an array of IMAPItem objects.
	 */
	public function getItems($aFilterTag)
	{
		$filter = $aFilterTag['sql'];
		$orderBy = $aFilterTag['orderby'];
		$offset = $aFilterTag['offset'];
		$limit = $aFilterTag['limit'];
		$fields = $aFilterTag['tag'];

		// always get RID attribute of message
		if ($fields && !strstr($fields, 'rid'))
			$fields .= ',rid';

		return parent::getItems('POP3Item', $filter, $orderBy, $offset, $limit,$fields);
	}

	/**
	  @brief Get an item.
	  @param $itemID ID of the item.
	  @return IMAPItem object.
	 */
	public function getItem($itemID, $cache = array())
	{

	  if (!$cache['cache']) $cache = false;
		return parent::getItem('POP3Item', $itemID, $cache);
	}

	/**
	  @brief Create a new item.
	  @param $message The message of the item.
	 */
	public function createItem($message)
	{
		return POP3Item::create($this, $message);
	}
	
	public function markItems($flag)
	{
    parent::markItems($flag);
  }
  
  public function unmarkItems($flag)
  {
    parent::unmarkItems($flag);   
  }

	/**
	  @brief Synchronize the item cache with the IMAP server.
	 */
	public function sync()
	{

		$account = $this->account;

    //Create instances of imap and cache classes
		$pop3 = POP3::instance($account);
    

    $aItemsID = $pop3->getItemsID($this->name);

    //Create temporarily unavailable
    file_put_contents($_SESSION['USERDIR'].SETTINGS_FOLDER.UNAVAILABLE_FILE,1);
    $oldExecutionTime = ini_alter('max_execution_time',MAX_EXECUTION_TIME);

    parent::sync($this->itemClassName,$aItemsID,$this);

    //Delete temporarily unavailable 
    @unlink($_SESSION['USERDIR'].SETTINGS_FOLDER.UNAVAILABLE_FILE);
    ini_set('max_execution_time',$oldExecutionTime);
	}
}

?>
