<?php

define('S_FOLDER_NAME','SPAM_QUEUE');
define('Q_FOLDER_NAME','Quarantine');
define('B_FOLDER_NAME',S_FOLDER_NAME.'/Blacklist');
define('W_FOLDER_NAME',S_FOLDER_NAME.'/Whitelist');


//FIXMErequire_once('inc/include.php');


require_once('inc/system.php');
require_once('inc/xml.php');
require_once('inc/xmlrequest.php');
require_once('inc/account.php');

require_once('inc/storage/storage.php');
require_once('inc/webmailiqprivate.php');
require_once('inc/localpop/account.php');
require_once('inc/pop3/account.php');
require_once ('inc/quarantine/folder.php');

/* Switch off cookies sessions. */
ini_set('session.use_cookies', '0');

/**
  @class User
  @brief Class representing an user.
  @author Martin Zlomek
  @date 21-02-2006
 */
class User {
	/** List of accounts. */
	public $aAccounts;

	/** ID of the user. */
	public $userID;

	/** Name of the user. */
	public $username;

	/** User's password */
	public $password;

	/** User's e-mail*/
	public $email;

	//Merak API objects
	public $mAccount;
	public $mAPI;
	
	/** List of uploaded attachments. */
	private $attachments;

	/**
	  @brief Create a new user object.
	  @param $userID ID of the user.
	  @param $username name of the user
	  @param $password user password
	  @param $hashid hashed login string
	  @param $mAccount logged MerakAccount object 
	 */
	protected function __construct($userID, $username, $password, $hashid, $mAccount)
	{
		global $aError;

		$this->mAPI	 = new MerakAPI();
		$this->mAccount = $mAccount;

		// Store important data 
		$this->aAccounts = array();
		$this->userID	= $userID;
		$this->username  = $username;
		$this->password  = $password; 
		$this->email	 = $this->mAccount->EmailAddress;  

		$this->attachments = array();

		// Get additional data like system paths and usertype 
		$this->GetData();

    // Delete unavailable.tmp file after login
    $filename = $this->getDir().UNAVAILABLE_FILE;
      if(file_exists($filename))
        unlink($filename);

		// Load user's accounts 
		$this->loadAccounts($_SESSION['MAILBOX'],$password,$this->mAccount);
    
    if(!$_SESSION['LAST_LOGIN_LOAD']){

      $oAccount = $this->getAccount($_SESSION['EMAIL']);
    
      $aProperties['current_time'] = date("Y/m/d H:i:s");
      $aProperties['current_ip'] = $_SERVER['REMOTE_ADDR'];
      
      if ($oAccount->last_time)
        $aProperties['last_time'] = $oAccount->last_time;
      if ($oAccount->last_ip)
        $aProperties['last_ip'] = $oAccount->last_ip;

      if (!$oAccount->last_cleanup || $oAccount->last_cleanup + (3*DAY_LENGTH) < time()){
          $aProperties['last_cleanup'] = time();
          $this->cleanUp();
      }
      
      $this->editAccount($_SESSION['EMAIL'],$aProperties); 
      
      $_SESSION['LAST_LOGIN_LOAD'] = true;
    }				
		
	}
	
	private function cleanUp(){

    $folder = $this->getDir();
    $this->mAPI->DeleteFiles($folder,'.msg',true,7);

  }

	/**
	  @brief Gets an system paths and user's email folder and his email addres
	 */
	private function getData()
	{
		/* API object to be used */
		$api  = &$this->mAPI;
		$user = &$this->mAccount;
				
		/* Store the system paths */
		$_SESSION['TEMPPATH']	 = $api->GetProperty("C_System_Storage_Dir_TempPath");
		$_SESSION['MAILPATH']	 = $api->GetProperty("C_System_Storage_Dir_MailPath");
		$_SESSION['LOGPATH']	 = $api->GetProperty("C_System_Storage_Dir_LogPath");
		
		$_SESSION['CONFIGPATH']	= $api->GetProperty("c_configpath");
		$_SESSION['INSTALLPATH']  = $api->GetProperty("c_installpath");
		$_SESSION['WMCONFIGPATH'] = WM_CONFIGPATH; 
  
		/* Store user's data */		  
		$_SESSION['USERDIR'] = $user->GetProperty("u_fullmailboxpath");
		$_SESSION['FULLNAME'] = $user->GetProperty("u_name");
    $_SESSION['ALTERNATIVE'] = $user->GetProperty("u_alternateemail");
    $_SESSION['SPAM_FOLDER'] = $user->getProperty("U_SpamFolderSupport");
    $_SESSION['PASSWORD_EXPIRED'] = $user->getProperty("U_PasswordExpired");
        
    if ($quota = $user->GetProperty("U_MailboxQuota"))
      $_SESSION['DISK_QUOTA']  = $quota;	
          
		// Get mailbox - u_mailbox if not login with email
		if (!$api->GetProperty("C_Accounts_Policies_Login_LoginSettings"))
         		$_SESSION['MAILBOX'] = $user->GetProperty("u_mailbox");	
		// Email if login with email
		else
         		$_SESSION['MAILBOX'] = $this->email;
         		
		/* Checks if the user is ADMIN(1), DOMAINADMIN(2) or USER(0) and stores this information */	 
		if($user->GetProperty("u_admin"))
			$_SESSION['ACCOUNT'] = 1;
		elseif(allowdadmin && $user->GetProperty("u_domainadmin"))
			$_SESSION['ACCOUNT'] = 2;
		else
			$_SESSION['ACCOUNT'] = 0;	  
      
        
    //REquest uid	
		$_SESSION['REQUEST_UID'] = time();

    //Server timezone
		$_SESSION['TIMEZONE']  = $api->GetProperty("c_timezone");	

		
    //Load settings
    self::settingsGlobal();
    self::settingsUser();

    //CheckDB
		if (!$_SESSION['DB_CHECKED'])
      $_SESSION['DB_CHECKED'] = self::checkDB();
      

  }
  public static function settingsGlobal()
  {   
      if(!$_SESSION['SETTINGS_GLOBAL'])
      {
      if(!file_exists(GLOBAL_SETTINGS_FILE)){
        $fp = fopen(GLOBAL_SETTINGS_FILE,"w+");
        fwrite($fp,file_get_contents(GLOBAL_SETTINGS_DEFAULT));
        fclose($fp);
      }
      
      $data = file_get_contents(GLOBAL_SETTINGS_FILE);
      
      $xml = XML::openFromString(trim($data));
      $settings = Tools::makeTreeFromXML($xml);

      if (!$settings['@childnodes']['global_settings']){
        
        $fp = fopen(GLOBAL_SETTINGS_FILE,"w+");
        fwrite($fp,file_get_contents(GLOBAL_SETTINGS_DEFAULT));
        fclose($fp);
        
        $xml = XML::openFromString(trim($data));
        $settings = Tools::makeTreeFromXML($xml);      
      }
      
      $set = $settings['@childnodes']['global_settings'][0]['@childnodes']['item'][0]['@childnodes'];
      
      $_SESSION['DBCONN'] = $set['dbconn'][0]['@value'];
      $_SESSION['DBUSER'] = $set['dbuser'][0]['@value'];
      $_SESSION['DBPASS'] = $set['dbpass'][0]['@value'];
      
      if(!$_SESSION['DBCONN']) 
        $_SESSION['DBCONN'] = 'sqlite:%WMUSERPATH%cache.db';
      
      $arr = explode(":",$_SESSION['DBCONN']);
      $_SESSION['DBTYPE'] = strtoupper($arr[0]);
      
      if($arr[1]) $_SESSION['PURECONN'] = $arr[1];
      
      $_SESSION['HOST'] = $set['smtpserver'][0]['@value']?$set['smtpserver'][0]['@value']:$_SERVER['SERVER_NAME'];
      $_SESSION['PORT'] = $set['smtpport'][0]['@value'];

      $_SESSION['IMAPHOST'] = $set['imapserver'][0]['@value']?$set['imapserver'][0]['@value']:$_SERVER['SERVER_NAME'];
      $_SESSION['IMAPPORT'] = $set['imapport'][0]['@value']; 
                  
      $_SESSION['LOGGING_TYPE'] = $set['logging_type'][0]['@value'];
      $_SESSION['LOGS'] = $set['logs'][0]['@value'];
      
      $_SESSION['SETTINGS_GLOBAL'] = true;
    }
}
  
  public static function settingsUser(){

  if (!$_SESSION['SETTINGS_USER']){ 

  //  $userdata = Storage::getUserData('settings'); 
		$domaindata = Storage::getDomainDefaults('settings',false);
		$defaultdata = Storage::getDefaults('settings');
		$data = WebmailIqPrivate::get($defaultdata,$domaindata,false);

    $restrictions = trim(strtoupper($data['@childnodes']['restrictions'][0]['@childnodes']['item'][0]
                                    ['@childnodes']['disable_gw_types'][0]['@value']));
 
    for($i = 0;$i < strlen($restrictions);$i++)
      $_SESSION['RESTRICTIONS'][strval($restrictions[$i])] = true;
    

    $_SESSION['DEFAULTCHARSET'] = $data['@childnodes']['mail_settings_default'][0]['@childnodes']['item'][0]
                                  ['@childnodes']['charset'][0]['@value'];
    $_SESSION['SETTINGS_USER'] = true;
    }
  }
  
	/**
	  @brief Load the list of accounts from the configuration file.
	 */
	public function loadAccounts($user,$pass,$mAccount)
	{
	

		$primaryProtocol = ($mAccount->getProperty("U_AccountType")==0)?'local':'imap';
		$email = $mAccount->EmailAddress;
	  $server = $_SESSION['IMAPHOST'];
	  $port = $_SESSION['IMAPPORT'];

		// Append the primary account to the list of accounts
		$primaryProperties = array('accountID' => $email, 
							'gw' => true, 
							'protocol' => $primaryProtocol, 
							'server' => $server, 
							'port' => $port, 
							'username' => $user, 
							'password' => $pass,
							'acc_type' => $_SESSION['ACCOUNT'],
							'alternative' => $_SESSION['ALTERNATIVE']); 
		
    $this->protocol = $primaryProtocol;

		// Open the configuration file with accounts. 
		$doc = XML::open($this->getDir() . accountsfile);

		if (!is_object($doc)) {

		  $primaryProperties['fullname'] = $_SESSION['FULLNAME'];
			// just append the primary account
			$this->appendAccount($primaryProperties, true);

			return;
		}
					
	  $count = 0;
		$otherProperties = array();
		foreach ($doc->query('/accounts/account') as $accountNode) {
			

      $aID = $accountNode->getAttribute('id');
      
      //FIX primary account - the first one in accounts.xml
      if($count++==0 && $email!=$aID){
			 $accountNode->setAttribute('id',$email);
			 $aID = $email;
       $doc->save();
      }
      			
		  $primary = ($aID == $email)?true:false;			 
      $gw = ($accountNode->getAttribute('gw')=='true')?true:false;
 	
       if (isset($this->aAccounts[$aID]))
					continue;

			//Primary account
			if ($primary) {
			    $sFullname = $doc->getNodeValue('fullname', $accountNode);
          $primaryProperties['description'] = $doc->getNodeValue('description', $accountNode);
					$primaryProperties['fullname'] = $sFullname?$sFullname:$_SESSION['FULLNAME'];
          if (!$_SESSION['LAST_LOGIN_LOAD']){
					 $primaryProperties['last_time'] = $doc->getNodeValue('current_time', $accountNode);
        	 $primaryProperties['last_ip'] = $doc->getNodeValue('current_ip', $accountNode);
        	 $primaryProperties['last_cleanup'] = $doc->getNodeValue('last_cleanup', $accountNode);
          }
			} else {
			    $pass = false;
			    if ($mAccount->Open($aID))
            $pass = $mAccount->getProperty('u_password');
          
          if(!$pass || $pass=='*')
            $pass = $doc->getNodeValue('password', $accountNode);
          
          $otherProperties[] = array('accountID' => $aID,
                      								'protocol' => $doc->getNodeValue('protocol', $accountNode), 
                      								'server' => $doc->getNodeValue('server', $accountNode), 
                      								'port' => intval($doc->getNodeValue('port', $accountNode)),
                      								'description' => $doc->getNodeValue('description', $accountNode), 
                      								'fullname' => $doc->getNodeValue('fullname', $accountNode), 
                      								'username' => $doc->getNodeValue('username', $accountNode), 
                      								'gw'=>$gw, 
                      								'password' => $pass);
			}
		}
		
    if (!$primaryProperties['password'])
     $primaryProperties['password'] = $_SESSION['user']->password;

		/* Append the accounts to the list of accounts. */
		$this->appendAccount($primaryProperties, true);
    foreach ($otherProperties as $properties)
			$this->appendAccount($properties, false);
    	

	}

  public static function checkDB()
  {
    if($_SESSION['DBTYPE']=="SQLITE"){
        $path = self::getDir();
        
        $cacheFile = str_replace("%WMUSERPATH%",$path,$_SESSION['PURECONN']);       

        if (file_exists($cacheFile)) {

          $check = new PDO("sqlite:".$cacheFile);
        
          $check->beginTransaction();
          
          $stmt = $check->prepare("SELECT item_value FROM wm_metadata WHERE item_key = ? and item_value >= ?");
          if ($stmt) $stmt->execute(array(0=>'db_version',1=>WEBMAIL_DB_VERSION));
        
          $check->commit();

          if ($stmt) $data = $stmt->fetch();

          unset($stmt);
          unset($check);
    
      	 if (!$data) @unlink($cacheFile);
        }
    }
  
  	return true;	
  }


	/**
	  @brief Login an user.
	  @param $username Name of the user.
	  @param $password Password of the user.
	  @param $hashid ID for the cryptographic hashing.
	  @return Session ID.
	 */
	public static function login($username, $password, $hashid = "", $method = "",$bSaveLoginTime = true)
	{
		global $aError;
	


		/* Create a new session. */
		session_start();

		/* Process user login. */
		$mAccount = new MerakAccount();
		
		/* Hadhid and challenge string - SHA1 */
		//$hashid = "SHA1|".$hashid;

    //TODO LOGGING EMAIL ADDRESS
    self::settingsGlobal();
    
    $bLogType = $_SESSION['LOGGING_TYPE']==true?true:false;


    /* Login /w or without hash  */
		//if (!$hashid){
     $auth = $mAccount->AuthenticateUserHash($username, $password, $_SERVER['SERVER_NAME'],'',$bLogType);
		/*}else {
    
      $auth = $mAccount->AuthenticateUserHash($username, $password, $_SERVER['SERVER_NAME'], $hashid, $bLogType);
	  }*/
		if (!$auth)
			throw new Exc($aError['invalid_login'],'invalid_login');


    //Check if expiration is enabled
    /*if (intval($mAccount->GetProperty('U_AccountValid')))
    {
      //Check if account is expired 
      $till =  $mAccount->GetProperty('U_AccountValidTill') + UNIX1900;
      $date = MerakGWAPI::unix2calendarDate(time());
		  
		  if ($till < $date)
		    throw new Exc($aError['account_expired'],'account_expired');
		}
		
    if (intval($mAccount->GetProperty('U_AccountDisabled')))
			throw new Exc($aError['account_disabled'],'account_disabled');
		*/
			
		if (!$mAccount->ValidateUser())
			throw new Exc('u_login_accountvalid','u_login_accountvalid');
		
		/* Log the user in and store the information to the session. */
		$_SESSION['EMAIL'] = $mAccount->EmailAddress;
		$_SESSION['DOMAIN'] = substr($_SESSION['EMAIL'],strpos($_SESSION['EMAIL'],'@')+1);

    $user = new User(/* FIXME: */0, $username, $password, $hashid, $mAccount);
    
		$_SESSION['user'] = $user;

	  //Remove directory 
	  $user->removeUploadDir();
    
		return session_id();
	}

	/**
	  @brief Logout the user.
	 */
	public function logout()
	{
		//Logout from GW 
		foreach($this->aAccounts as $account)
		  if ($account->gwAccount)
        $account->gwAccount->logout();
          
		$_SESSION = array();
		if (isset($_COOKIE[session_name()])) {
		  setcookie(session_name(), '', time()-42000, '/');
		}
		@session_destroy();
	}

	/**
	  @brief Load the user session.
	  @param $sid ID of the session.
	  @return User object.
	 */
	public static function load($sid)
	{
		/* Load the session. */
		session_id($sid);
		session_start();

		/* Check if the user is logged in. */
		if (session_id() !== $sid)
		{
			throw new Exc('User is not logged in - current session id: ' .session_id()
				. ', used session id: ' . $sid,'session_expired');
		} 
		else if (!isset($_SESSION['user']))
		{
			throw new Exc('User is not logged in - no session user set','session_no_user');
		}
		$_SESSION['REQUEST_UID'] = time() + microtime();

		return $_SESSION['user'];
	}

	/**
	  @brief Get user's directory.
	 */
	public static function getDir($root = false)
	{
		return $_SESSION['USERDIR'] . ($root ? '' : SETTINGS_FOLDER);
	}
	
	/**
	  @brief Get folder for uploading files.
	  @param $folder Folder ID
	  @return Full name of the folder for uploads
	 */
	private function getUploadDir($folder)
	{
		$root = $this->getDir() . '~upload';
		if (!is_dir($root))
			System::mkdir($root);
		
		$dir = $root. '/' . $folder;
		if (!is_dir($dir))
			System::mkdir($dir);
		
		return $dir;
	}
	
	/**
	  @brief Remove temporary folder for uploading files.
	  @param $folder Folder ID
	 */
	private function removeUploadDir($folder = '')
	{
	  if($folder) $folder = '/'.$folder;
		System::rmdir($this->getDir() . '~upload' . $folder);
	}

	/** 
	  @brief Get the list of accounts.
	  @return The list as an array of Account objects.
	 */
	public function getAccounts()
	{
		return $this->aAccounts;
	}

	/**
	  @brief Get an account.
	  @param $accountID ID of the account.
	  @return Account object.
	 */
	public function getAccount($accountID)
	{
	  global $aError;
		if (isset($this->aAccounts[$accountID]))
			return $this->aAccounts[$accountID];
		else
			throw new Exc(sprintf($aError['u_missing_account'],$accountID),'u_invalid_account');
	}

	/**
	  @brief Create a new account.
	  @param $properties Associative array with account properties:
			  $gw Account-has-GroupWareAccount flag.
			  $name Name of the account.
			  $description Description of the account.
			  $protocol Account protocol.
			  $server Name of the server where the account is located.
			  $port Port where connect to on the server.
			  $username Account logging username.
			  $password Account logging password.
	 */
	public function createAccount($properties)
	{
		/* Open the configuration file with accounts. */
		$file = $this->getDir() . accountsfile;
		$doc = XML::open($file);
		if (!is_object($doc)) {
			$doc = new XML($file);
			$accountsNode = $doc->appendChild(new DOMElement('accounts'));
		} else
			$accountsNode = $doc->getNode('/accounts');


		/* Get ID of the account. */
		$accountID = $properties["email"];

		$properties['accountID'] = $accountID;
		$properties['gw'] = false;

		/* Append the account to the configuration file. */
		$accountNode = $accountsNode->appendChild(new DOMElement('account'));
		
		$accountNode->setAttribute('id', $accountID);
		$accountNode->setAttribute('gw', $properties['gw'] ? 'true' : 'false');
		$accountNode->setAttribute('primary', 'false');
    		
		$accountNode->appendChild(new DOMElement('name', XML::encode($properties['name'])));
		$accountNode->appendChild(new DOMElement('description', XML::encode($properties['description'])));
		$accountNode->appendChild(new DOMElement('fullname', XML::encode($properties['fullname'])));
		$accountNode->appendChild(new DOMElement('protocol', XML::encode($properties['protocol'])));
		$accountNode->appendChild(new DOMElement('server', XML::encode($properties['server'])));
		$accountNode->appendChild(new DOMElement('port', $properties['port']));
		$accountNode->appendChild(new DOMElement('username', XML::encode($properties['username'])));
		$accountNode->appendChild(new DOMElement('password', XML::encode($properties['password'])));	
		
    $doc->save($file);	
		
		/* Append the account to the list of accounts. */
		$this->appendAccount($properties, false);

	}

	/**
	  @brief Delete an account.
	  @param $accountID ID of the account.
	 */
	public function deleteAccount($accountID)
	{

    $oAccount = $this->getAccount($accountID);

    //Delete cache records
    $oAccount->cleanUpCache();

		/* Open the configuration file with accounts. */
		$doc = XML::open($this->getDir() . accountsfile);
		if (!is_object($doc))
			return;

		/* Delete the account from the configuration file. */
		$accountNode = $doc->getNode('/accounts/account[@id="' . $accountID . '"]');
		$accountNode->parentNode->removeChild($accountNode);

    
		$doc->save();
		
		//Delete cache folder
    System::rmdir($this->getDir()."~".$accountID);
    

    
    //Delete account from session
		unset($this->aAccounts[$accountID]);
	}

	/**
	  @brief Edit properties of an account.
	  @param $accountID ID of the account.
	  @param $properties Properties as an array of (property => value).
	 */
	public function editAccount($accountID, $properties)
	{
		/* Update account internals. */
		$account = $this->getAccount($accountID);
		$account->edit($properties);

		/* Open the configuration file with accounts. */
		$doc = XML::open($this->getDir() . accountsfile);

    if (!is_object($doc)){
			
      $doc = new XML();
      $accountsNode = $doc->createElement("accounts");

			$accountNode = $doc->createElement("account");
			$accountNode->setAttribute("id",$accountID);
			$accountsNode->appendChild($accountNode);
			$doc->appendChild($accountsNode);
    } else{
      $accountNode = $doc->getNode('/accounts/account[@id="' . $accountID . '"]');
    }
		/* Update the configuration file. */

    if($primary){
      unset($properties['fullname']);
      unset($properties['password']);
    }

		foreach ($properties as $property => $value) {
			if ($propertyNode = $doc->getNode($property, $accountNode)){
			 $propertyNode->nodeValue = XML::encode($value);
      }else{
			 $propertyNode = $doc->createElement($property,XML::encode($value));
			 $accountNode->appendChild($propertyNode);
		  }
    }

		$doc->save($this->getDir() . accountsfile);


		$this->aAccounts[$accountID] = $account;
	}

	/**
	  @brief Create specific account object and append it to the list of accounts.
	  @param $properties Associative array with account properties:
				$accountID ID of the account.
			  $gw Account-has-GroupWareAccount flag.
			  $protocol Account protocol.
			  $server Name of the server where the account is located.
			  $port Port where connect to on the server.
			  $username Account logging username.
			  $password Account logging password.
	 */
	public function appendAccount($properties, $primary)
  {
    $account = false;
		switch (strtolower($properties['protocol'])) {
		case  'local':
		  $account = new LocalPOPAccount($this,$properties['accountID'],$primary,$properties);
		  
      break;
		case 'pop3':
      $account = new POP3Account($this, $properties['accountID'],
										$properties['server'], $properties['port'],
										$properties['username'], $properties['password'], $primary,$properties);  		

    break;
    case 'imap':
			$account = new IMAPAccount($this, $properties['accountID'],
										$properties['server'], $properties['port'],
										$properties['username'], $properties['password'], $primary,$properties);
			break;
  
		break;

		default:
			throw new Exc('Unknown account protocol');
		}

		if ($properties['gw']==true) {
			/* Append GroupWare account. */
			if (DEBUG) 
      echo "Appending groupware accout\r";
      
      $gwAccount = new GroupWareAccount($properties['accountID'], $properties['password'], $account);
      $account->appendGroupWareAccount($gwAccount);
		}
      

		$account->description = $properties['description'];
		$account->fullname = $properties['fullname'];
		
		if ($primary) {
			$account->alternative = $properties['alternative'];
			$account->acc_type = $properties['acc_type'];
		}

		//Create quarantine folder
		//TODO : the condition C_AS_General_Enable should be changed to test whether the Quarantine is activated on Merak
		
		if ($primary && $this->mAccount->GetProperty('U_QuarantineSupport') == 1)
		  $account->folders['quarantine'][Q_FOLDER_NAME] = QuarantineFolder::create(Q_FOLDER_NAME,$account);
		
		
    if($primary && ($this->mAccount->GetProperty('U_QuarantineSupport') == 1 || $this->mAccount->GetProperty('U_ASSupport') == 1)){
		  $account->folders['quarantine'][B_FOLDER_NAME] = QuarantineFolder::create(B_FOLDER_NAME,$account);
		  $account->folders['quarantine'][W_FOLDER_NAME] = QuarantineFolder::create(W_FOLDER_NAME,$account);
    }
    $this->aAccounts[$properties['accountID']] = $account;
	}

	/**
	  @brief Get the list of uploaded attachments of a folder.
	  @param $folder Name of the folder.
	  @return The list as an array of ('file', 'type').
	 */
	public function getAttachments($folder = false)
	{
	  if ($folder)
    { 
		      if (!isset($this->attachments[$folder]))
			         return array();

		      return $this->attachments[$folder];
	  } else {
          return $this->attachments;
    }
  }

	/**
	  @brief Add an uploaded attachment to a folder.
	  @param $uploadFileID Name of the file in the _FI
	  @param $folder Name of the folder.
	  @param $itemID ID of the item in attachments array
	  @param True on success, false on failure.
	 */
	public function addAttachment($uploadFileID, $folder, $itemID,$cid = false)
	{
		/* Create target directory. */
		$dir = $this->getUploadDir($folder);
    
    //log_buffer($_FILES[$uploadFileID]['name'],'ERROR');
    
		/* Save the attachment. */
		$fName = basename($_FILES[$uploadFileID]['name']);

    $file = $dir . '/' . uniqid() . '.tmp';
		if (!move_uploaded_file($_FILES[$uploadFileID]['tmp_name'], $file))
			return false;
    
    if ($cid)
		  $this->attachments[$folder][$itemID] = array('file' => $file,
																					       'name' => $fName,
																					       'type' => $_FILES[$uploadFileID]['type'],
                                                 'cid'  => $cid);
    else
		  $this->attachments[$folder][$itemID] = array('file' => $file,
																					       'name' => $fName,
																					       'type' => $_FILES[$uploadFileID]['type']);
    
		return true;
	}

	/**
	  @brief Remove a folder of uploaded attachments.
	  @param $folder Name of the folder.
	 */
	public function removeAttachments($folder)
	{
		if (!isset($this->attachments[$folder]))
			return;

		unset($this->attachments[$folder]);

		$this->removeUploadDir($folder);
	}
	
}

?>
