<?php

require_once 'error.php';
require_once 'webmailiqfolders.php';
require_once 'webmailiqauth.php';

/**
 * @brief		Class for handling webmailIqAccounts namespace
 */
 class WebmailIqAccounts extends XMLRequestInterface{

		 public $sAccountID;

		/**
		* @brief Public: Constructor
		* @param $oDOMQuery A DOMElement filled with query element
		* @param $oDoc	A DOMDocument of the request
		* @param $attrs  An Array with parent attributes
		*/
		public function __construct($oDOMQuery,&$oDOMDoc,&$attrs)
		{
				$this->oDOMQuery = $oDOMQuery;
				$this->oDOMDoc = &$oDOMDoc;
				$this->aAttrs = &$attrs;

				$this->checkInputXML();
				$this->exeInputXML();

		}
		/**
		* @brief	Private - Checks if the input XML is valid
		* @return  TRUE/FALSE and throws exception
		*/
		 private function checkInputXML(){
				global $aError;

				if ($this->aAttrs['type'] == 'set') {
				
					/* Check account node existence */
					if (!$oDOMAccount = $this->oDOMDoc->getNode("accounts:account",$this->oDOMQuery))
					  throw new Exc($aError['missing account']);	
					
					/* Check if there is an correct action specified */
					$aActions = array('edit'=>1, 'add' =>1, 'delete'=>1,'sync'=>1,'refresh'=>1,'test'=>1);
					$this->sAction = $oDOMAccount->getAttribute("action");

					if (isset($aActions[$this->sAction]))
					{
					  if (($this->sAction=='add' || $this->sAction=="test")  && ($this->oEmail = $this->oDOMDoc->getNode("accounts:email",$oDOMAccount)))
					  {
						      $this->sAccountID = $this->oDOMDoc->getNodeValue("accounts:email",$oDOMAccount);
            }
            else { 
                  if (!$this->sAccountID = $oDOMAccount->getAttribute("uid")) 
								        throw new Exc($aError['a_missing_aid'],'a_missing_aid');
            }
            
            //FIXME:domeny bez tecky
					  //if (!Security::checkemail(strtolower($this->sAccountID)))
						//      throw new Exc($aError['invalid id']);
					}
					else
					  throw new Exc($aError['invalid action']);
				}
		 }
		/**
		* @brief	Private: Do operations dependent on 'query xmlns'
		* @return		XML subanswer (string)
		*/
		 private function exeInputXML(){
			global $aError;
											  
			// Retrieve data to work with 
			$oUser = &$_SESSION['user'];
			$aAccounts = &$oUser->aAccounts;

			// Account identificator
			$sAid	= $this->sAccountID;
			switch ($this->aAttrs['type']) {
			 case 'get':
				$this->sTemplateFile = 'webmailiqaccounts_get';

				/* convert accounts for template */
				$this->aData = $this->cnvAccounts($aAccounts);

				break;
			case 'set':
	      // Account input 
				$oDOMAccountinp = $this->oDOMDoc->getNode("accounts:account",$this->oDOMQuery);
		    $aProperties = Tools::makeArrayFromXML($oDOMAccountinp);	
		    	
						   
        switch ($this->sAction){
            //Add account
				    case 'add':
							if ($sAid == $oUser->email)	
								throw new Exc("WebmailIQAccounts : Account already exist","account_already_exists"); 
              

              // Set default values
              if (!$aProperties['username']) throw new Exc($aError['no_username'],'no_username');
				      if (!$aProperties['password']) throw new Exc($aError['no_password'],'no_password');
              if (!$aProperties['protocol']) $aProperties['protocol'] = 'imap';
				      if (!$aProperties['server'])   $aProperties['server'] = '127.0.0.1';
				      if (!$aProperties['port'])     $aProperties['port'] = '143';
				      
              //Create account in accounts.xml
							$oUser->createAccount($aProperties);
							
              $account = $oUser->getAccount($sAid);
              
              //Synchronize edited account
			        if (!$account->sync())
					      throw new Exc($aError['imap_sync'],'imap_sync');	
					    
            break;
						//Edit account
            case 'edit':

              //Check for existence
              if (!$aAccounts[$sAid])
                throw new Exc("missing_account");


              //Check for primarity
					    $primary = ($oUser->email == $sAid);
					    
              //Get account being edited
              $account = &$aAccounts[$sAid];
              							
              //Edit primary account - API 
							if ($primary){

                //If there is something to edit in API Account
								if ($aProperties['username'] || $aProperties['password'] || $aProperties['alternative'] || $aProperties['fullname']){
                  
                  $mUser = new MerakAccount();
                  $mUser->open($sAid);
      
                  //Account opened successflly
									if ($mUser) {
                    if ($aProperties['username']){
                    
                      $account->username = $aProperties['username'];
                      $mUser->SetProperty("u_name",$aProperties['username']);
                      $mUser->Save();
                      
                    }
                    //Edit password
                    if ($aProperties['password']){
                      $password = '';
                    
                      $privateKey = openssl_pkey_get_private(file_get_contents(WM_CONFIGPATH.'private.key'));	

					            openssl_private_decrypt(WebmailIqAuth::hexStringToBinary($aProperties['password']), $password, $privateKey);
                      openssl_private_decrypt(WebmailIqAuth::hexStringToBinary($aProperties['oldpassword']), $oldPassword, $privateKey);

                      if ($oUser->password!=$oldPassword)
                        throw new Exc('account_bad_password','bad_password');
                        
                      $aProperties['password'] = $password;
                      $account->password = $password;
                      
                      //Save password
                      $mUser->SetProperty("u_password",$aProperties['password']);
                      $mUser->Save();
                      
                      //Get username
                      $aAlias = explode(";",$mUser->getProperty("u_alias"));
                      $aProperties['username'] = $aAlias[0];

                      //Relog user in order IMAP & GW works(new password => new connection)
                      $this->aAttrs['sid'] = User::login($aProperties['username'],$aProperties['password'],"","",false);					  
                      
                      unset($aProperties['password']);
                      unset($aProperties['oldpassword']);                      
                
                    }
                    //Edit alternative Email
                    if ($aProperties['alternative']) {
											$mUser->SetProperty("u_alternateemail",$aProperties['alternative']);
                      $mUser->Save();
                      unset($aProperties['alternative']);
                    } 
										//Edit fullname
                    if ($aProperties['fullname']){
                      $mUser->SetProperty("u_name",$aProperties['fullname']);
                      $mUser->Save();
                      
                      $_SESSION['FULLNAME'] = $aProperties['fullname'];
                      unset($aProperties['fullname']);
                    }                            												 
									}
								}  											
              }

                
              //Edit account
              $oUser->editAccount($sAid, $aProperties);
              
              //Synchronize edited account
			        if (!$account->sync())
					      throw new Exc($aError['imap_sync'],'imap_sync');	
					  
              //Quarantine & Filter folders handling
              //NOTE : needs to be done after sync 
              if ($primary){
            
                //API objects
					      $mAccount = new MerakAccount();
					      $mAccount->open($oUser->email);   
            
                //Add quarantine folders to the edited account if primary & supports quarantine
                if ($mAccount->GetProperty('U_QuarantineSupport') == 1)
                  $account->folders['quarantine'][Q_FOLDER_NAME] = QuarantineFolder::create(Q_FOLDER_NAME,$account);
                
                //Add Black & White filters
                if($mAccount->GetProperty('U_QuarantineSupport') == 1 || $mAccount->GetProperty('U_ASSupport') == 1){
          		    $account->folders['quarantine'][B_FOLDER_NAME] = QuarantineFolder::create(B_FOLDER_NAME,$account);
          		    $account->folders['quarantine'][W_FOLDER_NAME] = QuarantineFolder::create(W_FOLDER_NAME,$account);
                }                     
              }
              
              unset($aProperties['gw']);
              unset($aProperties['type']);


            break;
            //Delete action
            case 'delete':
              if ($sAid == $oUser->email) 
							  throw new Exc($aError['a_pad']);
							 
              //Delete account 
						  $oUser->deleteAccount($sAid);
            break;
            // Synchronize account
						case 'sync':
            case 'refresh':
            
							if (isset($aAccounts[$sAid]))
								$oAccount = &$aAccounts[$sAid];
							else
								throw new Exc('Account does not exist: ' . $sAid);

							//Refresh account
							$oAccount->sync();
							
							//Empty unsubscribed folders 1st synchronization list
              $oAccount->aSyncedFolders = array();

							// Refresh info about messages in all Mail folders
							foreach($oAccount->folders["main"] as $oFolder)
                if ($oFolder->sync)
                  $oFolder->sync($this);
              
              //Return list of refreshed folders
							if ($this->sAction=='refresh') {
								$folders = $oAccount->getFolders();
								$this->aData['aid']  = $sAid;
								$this->aData['folder']['num'] = WebmailIqFolders::cnvFolders($folders);
								$this->sTemplateFile='webmailiqfolders_get';
							}
            break;
            case 'test':
              $aProperties['accountID'] = $aProperties['email']; 
              
              if(!$aAccounts[$aProperties['accountID']]){
                $oUser->appendAccount($aProperties, false);
              
                $oTestAccount = $oUser->getAccount($aProperties['accountID']);             
                if(!$oTestAccount->test()){
                  unset($oUser->aAccounts[$aProperties['accountID']]);
                  throw new Exc('test_account_failed','test_account_failed');
                }else{
                  unset($oUser->aAccounts[$aProperties['accountID']]);
                }
                
              }                      
            break;

        }
				break;
			}
		}
		/**
		 *  @brief Converts account array into template compatible data array
		 *  @param $accounts Array of account objects
		 *  @return accounts array in the form for template system
		 */
		 private function cnvAccounts($accounts,$noshared = false){
			  if (!$accounts || !is_array($accounts)) return;

        $aData = array();
				$mAccount = new MerakAccount();			
				foreach($accounts as $key => $oAccount){

          if ($noshared && $oAccount->shared) continue;
					// Create account item for the template
					$aAcc = array(); 
					$aAcc["id"] = $key;
					$aAcc["gw"] = $oAccount->isGroupWare($logged)?'true':'false';
					if (!$logged) $aAcc["gw"] = 'false';
					$aAcc["port"] = $oAccount->port;

					// FIXME  POP3 & LocalPOP vs. other account types
					$aAcc["protocol"] = strtolower($oAccount->protocol);
					$aAcc["username"] = $oAccount->username;
					
					$aAcc["current_time"] = $oAccount->current_time;
					$aAcc["current_ip"] = $oAccount->current_ip;
					$aAcc["last_time"] = $oAccount->last_time;
					$aAcc["last_ip"] = $oAccount->last_ip;

					if (!$oAccount->primary) 
						$aAcc["password"] = $oAccount->password;
					else {
						$aAcc["alternative"] = $oAccount->alternative;
						switch ($oAccount->acc_type) {
							case 1:
								$aAcc["acc_type"] = 'admin';
								break;
							case 2:
								$aAcc["acc_type"] = 'domainadmin';
								break;
							default:
								$aAcc["acc_type"] = 'user';
								break;
						}
					}
					$aAcc["server"] = $oAccount->server;
					$aAcc["primary"] = $oAccount->primary?'true':'false';
					if ($oAccount->primary && $_SESSION['PASSWORD_EXPIRED'])
					 $aAcc["pass_expired"] = 'true';
					
					$aAcc["not_primary"] = $oAccount->primary?false:true;
					$aAcc["fullname"] = $oAccount->fullname;
					$aAcc["description"] = $oAccount->description;
					

					if ($_SESSION['DISK_QUOTA'] && $mAccount->Open($oAccount->accountID)){
            $aAcc["mbox_quota"] = $_SESSION['DISK_QUOTA'];
            $size = $mAccount->GetProperty("U_MailboxSize");
            $aAcc["mbox_usage"] = $size?$size:'0';
          }

					if (!$aAcc["description"]) $aAcc["description"] = $oAccount->fullname;
					

					// Append item to account list
					$aData['accounts']['num'][$key] = $aAcc;
			  }
			  return $aData;
		 }
 }
?>
