<?php

require_once('inc/system.php');

/**
 * @brief		Class for handling webmailIqAccounts namespace
 */
class WebmailIqFolders extends XMLRequestInterface{
	
  public $aActions = array('edit'=>1, 'delete'=>1, 'add'=>1, 'sync'=>1, 'empty'=>1,'markasread'=>1,'markasunread'=>1);
	
	private $oDOMAccount; //DOM Element representing account element in XML request
	
	private $oDOMFolder;  //DOM Element representing folder element in XML request
	
	private $sAID;	 //String ID of account element
	
  private $sFID;	 //String ID of folder element
	
	private $sFAction; //String folder action
	
	  /**
		* @brief Public: Constructor
		* @param $oDOMQuery A DOMElement filled with query element
		* @param $oDOMDoc	A DOMDocument of the request
		* @param $attrs  An Array with parent attributes
		*/
		public function __construct($oDOMQuery,&$oDOMDoc,&$attrs)
		{
			$this->oDOMQuery = $oDOMQuery;
			$this->oDOMDoc = &$oDOMDoc;
			$this->aAttrs = &$attrs;

				// Retrieve list of user's accounts 
		  	$this->aAccounts = &$_SESSION['user']->getAccounts();
		  	
			$this->checkInputXML();
		  	$this->exeInputXML();

	  	}
	  /**
		* @brief	Private - Checks if the input XML is valid
		* @return  TRUE/FALSE and throws exception
		*/
	  	private function checkInputXML(){
	  	global $aError;
	  	  
		    // Check if there is an account element in query 
	  	  if (!$this->oDOMAccount = $this->oDOMDoc->getNode("folders:account",$this->oDOMQuery))
	  		     throw new Exc($aError["missing account"]);
	  	 
	  	  // Check if an account id is set 
	  		if (!$this->sAID = $this->oDOMAccount->getAttribute("uid"))
	  				 throw new Exc($aError["missing aid"]);

	  		// Check if there is an account with specified id 
	  		if (!isset($this->aAccounts[$this->sAID]))
	  				 throw new Exc($aError["invalid aid"]);
	  		
	  		// Conditions for iq type='set' 
	  		if ($this->aAttrs['type']=='set'){
	  		
      	  		// Check for folder element existence 
      	  		if (!$this->oDOMFolder = $this->oDOMDoc->getNode("folders:folder",
                                                               $this->oDOMAccount))
                    throw new Exc($aError["missing folder"]);
      	  				
      			  // Check for folder action attribute existence 
      		  	if (!$this->sFAction = $this->oDOMFolder->getAttribute("action"))
      	  					throw new Exc($aError["missing action"]);  
      			  		
      	  				 
      	  		// Check for folder  type element existence 
      		  	if (!($this->sFType = $this->oDOMDoc->getNodeValue("folders:type",
                                                                 $this->oDOMFolder))
              && !$this->sFAction=='add')
      	  					throw new Exc($aError["missing type"]);
      		  
      		  
      			  // Check for folder  id attribute existence 
      		  	if ((!$this->sFID = $this->oDOMFolder->getAttribute("uid")) 
                  && ($this->sFAction!='add'))
      	  					throw new Exc($aError["missing id"]);
 			
        			// Check if folder action attribute is correct 
        			if (!isset($this->aActions[$this->sFAction]))
        	  					throw new Exc($aError["invalid action"]);
        	  }
	  	}

	  /**
		* @brief	Private: Do operations dependent on 'query xmlns'
		* @return		XML subanswer (string)
		*/
		 private function exeInputXML(){
				global $aError;

				// Get account to worh with 
				$oUser = &$_SESSION['user'];
				$oAccount = &$oUser->getAccount($this->sAID);	 

				// Process actions depending on set/get
				switch ($this->aAttrs['type']) {
						case 'get':
                  //Get folders
									$folders = $oAccount->getFolders();

									$this->sTemplateFile = 'webmailiqfolders_get';
									$this->aData['aid']  = $this->sAID;
									$this->aData['folder']['num'] = $this->cnvFolders($folders);
									break;
						case 'set':
									
                  // Get the type of the folder. 
                  if ($this->sFAction == 'add') {
									     $sFolderType = strtoupper($this->sFType{0});
									} else {
									     $folders = $oAccount->getFolders();
										   if (!isset($folders[$this->sFID]))
											       throw new Exc('Folder does not exists: ' . $this->sFID);

                       $sFolderType = $folders[$this->sFID]->type;
									}
									
									// Do some actions in case the folder is from groupware or imap 	
									switch ($sFolderType) 
									{
										case 'M': // MAIL folder
											// do nothing
											break;
														
										case 'C': // CONTACT folder
										case 'E': // EVENTS folder
										case 'T': // TASKS folder
										case 'N': // NOTES folder
										case 'J': // Journal folder
                    case 'F': // FILES folder
											$oAccount = &$oAccount->gwAccount;
											if (!is_object($oAccount))
                        throw new Exc('','gw_init_error');
											$this->gwFolder = true;
											break;
													
										default:
											throw new Exc('Unknown folder type');
									}
									 

									switch ($this->sFAction){
									    // Synchronize folder
											case 'sync':
											       
														$oFolder = &$oAccount->getFolder($this->sFID);
														if (isset($oFolder->gw) && $oFolder->gw)
															throw new Exc($aError['f_sync_gw']);
														$oFolder->sync($this); 
														break;
											// Delete existing folder
											case 'delete':
														// Retrieve folder to work with
														$oFolder = &$oAccount->getFolder($this->sFID);

														if (DEBUG) echo "Deleting folder".$oFolder->type."=".$oFolder->name."\r";
														$oFolder->delete();
														
														if ($sFolderType == 'M')
														{
															//Synchronize account to uncache this new folder
															$oAccount->sync();
														}
														break;
											// Delete all items from a folder
											case 'empty':
														// Only email folders (except from INBOX) can be emptied
														if ($sFolderType != 'M')
															throw new Exc($aError['f_empty_non_mail']);
															
														// TODO: support for moving to different account
														$sDestinationAccount = $this->oDOMDoc->getNodeValue("folders:account",$this->oDOMFolder);
														$sDestinationFolder = $this->oDOMDoc->getNodeValue("folders:folder",$this->oDOMFolder);
														
														// INBOX cannot be deleted
														/*if ($this->sFID == 'INBOX' && !$sDestinationFolder)
															throw new Exc($aError['f_empty_non_mail']);*/

                            //Retrieve folder to work with
														$oFolder = &$oAccount->getFolder($this->sFID);

                            //Copy items to folder
														if ($sDestinationFolder) {
														  $oDestinationAccount = &$oUser->getAccount($sDestinationAccount);
															$oDestination = &$oDestinationAccount->getFolderWithAutoCreate($sDestinationFolder);
                              
                              $oFolder->moveItems($oDestination);
														}
														else
															$oFolder->deleteItems();
														break;
                      case 'markasread':
                      case 'markasunread':
                            $flag = Item::FLAG_SEEN;
														
                            if ($sFolderType != 'M')
															throw new Exc($aError['f_mark_non_mail']);
                            //Retrieve folder to work with
														$oFolder = &$oAccount->getFolder($this->sFID);
														
														if ($this->sFAction=='markasread')
                              $oFolder->markItems($flag);
                            else 
                              $oFolder->unmarkItems($flag);
                      break;								
											// Add folder
											case 'add':
														// Get folder name
														$sFolderName = $this->oDOMDoc->getNodeValue("folders:name",
																								                        $this->oDOMFolder);
														if (!$sFolderName) throw new Exc($aError['f_name_missing']);
														
														
														//Create folder
														$oAccount->createFolder($sFolderName,$sFolderType);
														
														//Synchronize account to cache this new folder
														if ($sFolderType == 'M')
															$oAccount->sync();
														break;
											//Edit folder
											case 'edit':
											
														//Retrieve folder to work with
														$type = 'main';
														$oFolder = &$oAccount->getFolder($this->sFID,$type);
                            //dmp($oFolder);die();
														 /* Get old and new name*/
														 $sFolderName = $oFolder->name;
														 $sNewFolderName = $this->oDOMDoc->getNodeValue("folders:name",
                                                                            $this->oDOMFolder);
 
														 // Check new folder name 
														 Security::checkpath($sNewFolderName);
                             
														  // Rename folder 
														 if ($sNewFolderName && ($sFolderName!=$sNewFolderName))	{	
                                $oFolder->rename($sNewFolderName);
														
                               // Rename GW folder as well,in order to keep 
                               // synchronization between IMAP and GW database   
  														 if ($sFolderType == 'M'){
  																try {
  																  if ($oAccount->gwAccount){
                                     $oGWFolder = $oAccount->gwAccount->getFolder($sFolderName);
  																	 $oGWFolder->rename($sNewFolderName); 
  																  }
                                  } catch (Exc $e) {
  																			// folder does not exist in GW, continue
  															  }
  																// Synchronize account to cache this new folder 
  																$oAccount->sync();
                                     
                               // Rename IMAP folder as well in order to keep 
                               // synchronization between IMAP and GW database */
                                } else {
                                      
                                     $type='main';
                                      
                                     try{
                                      $oIMAPFolder = $oAccount->account->getFolder($sFolderName,
                                                                                  $type);
                                      $oIMAPFolder->rename($sNewFolderName); 
                                     }catch (Exc $e)
                                     {}                                                                    
                               }
                             }
                             
                             if($sSubscription = $this->oDOMDoc->getNode("folders:subscribed",
                                                                            $this->oDOMFolder)){
                                $sSubscription = $this->oDOMDoc->getNodeValue("folders:subscribed", $this->oDOMFolder);
                                
                                $oFolder->setSubscription($sSubscription);
                             }
                             
                                
                                
														 break;
									}
									
									break;
				}
		 }
		/**
		 *  @brief Converts folder array into template compatible data array
		 *  @param $folders Array of folder objects
		 *  @return folders array in the form for template system
		 */		 
		 public static function cnvFolders($folders)
		 {  
		    //dmp($folders);
				if (!is_array($folders)) return;
				$aItems = array();
				foreach($folders as $k=>$v){
				  $aItm = array();

          // Process Mail folder 
				  if (!isset($v->gw) || !$v->gw) {
					      $aItm['rights'] = self::rightsToString($v->rights);		      
      					$aItm['recent']  = $v->countItems(Item::FLAG_SEEN, false); 
                $aItm['subscribed']	  = $v->sync?true:false; 
          
          // Process GW folder 
          } else {
					      $aItm['rights'] = $v->rights;
      					$aItm['recent']  = '0'; 
         }
          
          // Process both  
					$aItm['name']	  = $v->name;
					$aItm['id']		= $v->name;
					$aItm['type']	  =  $v->type;
					//$aItm['total']	 = $v->countItems();

					$aItems[] = $aItm;
				}

				return $aItems;
			 
		 }
		 
		 public static function rightsToString($rights)
		 {
        $sRights='';
        
        $sRights .=  ($rights & Folder::RIGHT_READ) ?'r':'';
        $sRights .=  ($rights & Folder::RIGHT_WRITE) ?'w':'';
        $sRights .=  ($rights & Folder::RIGHT_DELETE) ?'d':'';
        $sRights .=  (($rights & Folder::RIGHT_INSERT) || ($rights & Folder::RIGHT_WRITE) || ($rights & Folder::RIGHT_CREATE) )?'m':''; 
        
        return $sRights;
     }
 }
?>
