<?php

require_once 'error.php';

/**
 * @brief  class for handling webmailIqItems namespace
	@author Pavel Perna
	@date	04.04.2006
	@TODO	SET Items 
 */
 class WebmailIqFreeBusy extends XMLRequestInterface {
		
		/**
		* @brief Public: Constructor
		* @param $oQuery A DOMElement filled with query element
		* @param $oDoc	A DOMDocument of the request
		* @param $attrs  An Array with parent attributes
		*/
		public function __construct($oQuery,&$oDoc,&$attrs)
		{
				$this->oQuery = $oQuery;
				$this->oDoc = &$oDoc;
				$this->aAttrs = &$attrs;

				$this->checkInputXML();
				$this->exeInputXML();

		}	
		
	  /**
		* @brief	Private - Checks if the input XML is valid
		* @return  TRUE/FALSE and throws exception
		*/		
		private function checkInputXML()
		{
				global $aError;
				if (!$oUser = $this->oDoc->getNode("freebusy:user",$this->oQuery))
						throw new Exc($aError["fb_missing_user"]);	
						
				if (!$this->sEmail = $oUser->getAttribute("email"))
						throw new Exc($aError["fb_missing_email"]);	  
		
		}

	  /**
		* @brief	Private: Do operations dependent on 'query xmlns'
		* @return		XML subanswer (string)
		*/
		private function exeInputXML()
		{		
				switch($this->aAttrs["type"])
				{
					case "get":		  
						$this->sTemplateFile = 'webmailiqfreebusy_get';
						 
						// GroupWare API object
						$oGroupWareAPI = new MerakGWAPI;
						
						// From and to (1 month from present)
						$from = $oGroupWareAPI->Unixdate2Str(mktime());
						$to = $from + 30;
						
						// Get FreeBusy depending on specified email
						$sVersit = $oGroupWareAPI->FunctionCall("GetVFreeBusy",
															 $this->sEmail,
															 $from . '-' . $to);
						
						// Parse versit data into return array for template 
						$this->aData = $this->parseVersitObject($sVersit);
						break;
				}
				 
		}
	  
	  /**
		* @brief	Private: Do operations dependent on 'query xmlns'
		* @return		XML subanswer (string)
		*/  
		private function parseVersitObject($sVersit)
		{

				$aLines = explode("\r\n",$sVersit);
				
				// Get all freebusy items
				foreach ($aLines as $sLine){
					  $aLine = explode(":",$sLine);
					  $aLineType = explode(";",$aLine[0]);
										
					  if ($aLineType[0]=="FREEBUSY"){
										
							$aBusyType = explode("=",$aLineType[1]);
							$sBusyType = $aBusyType[1];
							$aTime = explode("/",$aLine[1]);
											
							$aItem["type"] = $sBusyType;
							$aItem["user"] = $this->sEmail;
							
							$aItem["start"] = $aTime[0];
							$aItem["finish"] = $aTime[1];		  
							
							//From and to time
							for ($i = 0; $i < 2; $i++)
							{
								  $sYear[$i] =  substr($aTime[$i],0,4);
								  $sMonth[$i] =  substr($aTime[$i],4,2);
								  $sDay[$i] =  substr($aTime[$i],6,2);
								  $sHours[$i] = $sMinutes[$i] = $sSeconds[$i] = 0;
								  
								  if (strpos($aTime[$i],"T")!=-1)
								  {
									  $sHours[$i] = substr($aTime[$i],9,2);	 
									  $sMinutes[$i] = substr($aTime[$i],11,2);  
									  $sSeconds[$i] = substr($aTime[$i],13,2);	
									  $sUnix[$i] = mktime($sHours[$i],
														  $sMinutes[$i],
														  $sSeconds[$i],
														  $sMonth[$i],
														  $sDay[$i],
														  $sYear[$i]);	
								  }
							}
							$aItem["start"] = $sUnix[0];
							$aItem["finish"] = $sUnix[1]; 

							$aItems["items"]["num"][] = $aItem;
					  }
				} 
				return $aItems;
		}
 }
		
?>
