<?php

require_once 'error.php';

/**
 * @brief  class for handling webmailIqGroups namespace
	@author Pavel Perna
	@date	04.05.2006
	@TODO	All 
 */
 class WebmailIqGroups extends XMLRequestInterface {
		
		private $sAID; //Groupware account id(email adress)
		
		private $oAccount; //Link to groupware account
		
		private $aActions = array("edit"=>1, "delete"=>1, "add"=>1, "subscribe"=>1, "export"=>1,"import"=>1);
		
		/**
		* @brief Public: Constructor
		* @param $oQuery A DOMElement filled with query element
		* @param $oDoc	A DOMDocument of the request
		* @param $attrs  An Array with parent attributes
		*/
		public function __construct($oQuery,&$oDoc,&$attrs)
		{
				$this->oQuery = $oQuery;
				$this->oDoc = &$oDoc;
				$this->aAttrs = &$attrs;

				$this->checkInputXML();
				$this->exeInputXML();

		}	
		
	  /**
		* @brief	Private - Checks if the input XML is valid
		* @return  TRUE/FALSE and throws exception
		*/		
		private function checkInputXML()
		{
				global $aError;
				//Pointer to GW Account
				$this->pGWAccount = &$_SESSION['pGWAccount'];
				
				if (!is_object($this->pGWAccount))
					  throw new Exc("g_missinggwaccount");
						
				if (!$this->pGWAccount->gwAPI->IsConnected())
				{
					  $this->pGWAccount->gwAPI->Login();
					  $this->pGWAccount->gSID = $this->pGWAccount->gwAPI->OpenGroup("*");
				}			

				
				//Pointer to GW API
				$this->pGWAPI = &$this->pGWAccount->gwAPI;
				
				/**
				 *Check if user wants to retrieve list of subscribed groups
				 *or the list of groups that are available to subscribe to.
				 */
				if ($oGlobal = $this->oDoc->getNode("groups:global",$this->oQuery))
					  $this->sFunction = 'GetGlobalGroupList';
				else  $this->sFunction = 'GetGroupList';
					  
				if ($this->aAttrs['type']=='set')
				{
					  	/* Get group element */
						if (!$oGroup = $this->oDoc->getNode("groups:group",$this->oQuery))
									  throw new Exc($aError["invalid action"]);		
									  
						/* Check for group action attribute existence */
		  				if (!$this->sGAction = $oGroup->getAttribute("action"))
	  								throw new Exc($aError["missing action"]);
	  							
	  						  /* Check if group action attribute is correct */
						if (!isset($this->aActions[$this->sGAction]))
									  throw new Exc($aError["invalid action"]);
								
								/* If importing data check if there are any */
							  switch ($this->sGAction)
						{
							  case 'import':
					  					if(!$oData = $this->oDoc->getNode("groups:data",$oGroup))
					  						 throw new Exc($aError["g_data_missing"]);
					  					$this->sImport = $this->oDoc->getNodeContent($oData);
					  					  break;
					  				case 'subscribe':
							  case 'delete':
									if(!$this->sGID = $oGroup->getAttribute("uid"))
										throw new Exc($aError["g_id_missing"]);
									break;
							  case 'add':					  
											/* Create parameters */	 
									foreach ($oGroup->childNodes as $key=>$value)
										$this->sParameters.= $value->tagName.'='.$this->oDoc->getNodeValue("groups:".$value->tagName,$oGroup).'&';	
											
											
											/* Remove last char(&) and if there are no values log warning */
									if (!$this->sParameters) LogError::Error($aError["g_no_values"]);
									
									/* Add owner's email for reminder and daily event's info */
									$this->sParameters.='&grpdailyeventsemail='.$_SESSION['user']->email.'&grpreminderemail='.$_SESSION['user']->email;
	  									  break;
	  						  }
				}

						
								
		}

	  /**
		* @brief	Private: Do operations dependent on 'query xmlns'
		* @return		XML subanswer (string)
		*/
		private function exeInputXML()
		{		
				
				switch($this->aAttrs["type"])
				{
					case 'get':		  
						$this->sTemplateFile = 'webmailiqgroups_get';
						
						$sGroupLine = $this->pGWAPI->FunctionCall($this->sFunction,
																  $_SESSION['sGWSessionId']);					
						$aGroups = $this->pGWAPI->ParseParamLine($sGroupLine);
						
						$this->aData["group"]["num"] = $this->cnvGroups($aGroups);
						break;		
					case 'set':
						//$this->sTemplateFile = 'webmailiqgroups_set';
						switch ($this->sGAction)
						{				 
							  case 'add':
									if(!$this->pGWAPI->FunctionCall("AddGroup",
																	$this->pGWAPI->sessid,
																	$this->sParameters))
										  throw new Exc("g_add");
									break;
							  case 'subscribe':
									$this->sParameters = "&GrpLink=".$this->sGID;
									if(!$this->pGWAPI->FunctionCall("AddGroup",
																	$this->pGWAPI->sessid,
																	$this->sParameters))
										  throw new Exc("g_subscribe");
									break;
							  break;
							  case 'export':
									if (!$sExport = $this->pGWAPI->FunctionCall("ExportGroup",
																				$this->oGWAccount->gSID));
										  throw new Exc("g_export");
									break;
							  case 'import':
									if (!$this->pGWAPI->FunctionCall("ImportGroup",
																	 $this->oGWAccount->gSID,
																	 $this->sImport))
										  throw new Exc("g_import");
									break;
							  case 'delete':
									if (!$this->pGWAPI->FunctionCall("DeleteGroup",
																	 $this->pGWAPI->sessid,
																	 $this->sGID))
										  throw new Exc("g_delete");
									break;
						}
				}
				 
		}
	  /**
		*  @brief Converts groups array into template compatible data array
		*  @param $groups Array of groups
		*  @return groups array in the form for template system
		*/			
		private function cnvGroups($groups){
			 if (!is_array($groups)) return array();
			 $aItems = array();
		  foreach($groups as $group){
			$aItem["id"] = $group["GRP_ID"];
			$aItem["sFields"] = Tools::makeXMLTags($group);
			
			/* Open group and check for its rights */
			$gsid = $this->pGWAPI->OpenGroup($group["GRP_ID"]);			
			$aItem["rights"] = $this->pGWAPI->GetGroupRights($gsid);  
			$this->pGWAPI->CloseGroup($gsid);
			
			$aItems[] = $aItem;
		  }
		  return $aItems;
		}	  
}
		
?>


