<?php

require_once 'error.php';

/**
 * @brief  class for handling webmailIqimport namespace
	@author Pavel Perna
	@date	04.05.2006
	@TODO	All 
 */
 class WebmailIqImport extends XMLRequestInterface {
		
		private $sAID;	 //Groupware account id(email adress)
		
		private $oAccount; //Link to groupware account
		
		private static $aActions = array("csv"=>1, "vcard"=>1,"vcalendar"=>1,"ldif"=>1,"groupware"=>1);
		
		/**
		* @brief Public: Constructor
		* @param $oQuery A DOMElement filled with query element
		* @param $oDoc	A DOMDocument of the request
		* @param $attrs  An Array with parent attributes
		*/
		public function __construct($oQuery,&$oDoc,&$attrs)
		{
				$this->oDOMQuery = $oQuery;
				$this->oDOMDoc = &$oDoc;
				$this->aAttrs = &$attrs;

				$this->checkInputXML();
				$this->exeInputXML();

		}	
		
	  /**
		* @brief	Private - Checks if the input XML is valid
		* @return  TRUE/FALSE and throws exception
		*/		
		private function checkInputXML()
		{
				global $aError;	 

				if (!$this->oDOMImport = $this->oDOMDoc->getNode("import:import",$this->oDOMQuery))
					  throw new Exc("missing_import_tag","missing_import_tag");
				
				if (!$this->sFullPath = $this->oDOMDoc->getNodeValue("import:fullpath",$this->oDOMImport))
					  throw new Exc("missing_fullpath_tag","missing_fullpatg_tag");

        if($this->aAttrs["type"]=='set'){
				  $this->sSkipFirst = $this->oDOMDoc->getNode("import:skipfirst",$this->oDOMImport)?true:false;
		      
          // Check if there is an account element in query 
	  	    if (!$this->sAction = strtolower($this->oDOMImport->getAttribute("action")))
              throw new Exc($aError["missing action"],'missing_action');		      
		      
		      if(!self::$aActions[$this->sAction])
		          throw new Exc($aError["invalid action"],'invalid_action');
		      
          // Check if there is an account element in query 
	  	    if (!$this->oDOMAccount = $this->oDOMDoc->getNode("import:account",
                                                          $this->oDOMImport))
              throw new Exc($aError["missing account"],'missing_account');
              
          // Check if an account id is set 
	  		  if (!$this->sAID = $this->oDOMAccount->getAttribute("uid"))
	  			    throw new Exc($aError["missing aid"]);
	  			    
  	  		// Check if there is a account with specified id 
  	  		if (!$this->oAccount = $_SESSION['user']->getAccount($this->sAID))
  	  			    throw new Exc($aError["invalid aid"]);	  		
          
          switch($this->sAction){  	    
      	  	case 'csv':
            case 'ldif':
            case 'sif':
            case 'vcard':
            case 'vcalendar':
              // Check for folder element existence 
      	  		if (!$this->oDOMFolder = $this->oDOMDoc->getNode("import:folder",
                                                               $this->oDOMAccount))
      	  			    throw new Exc($aError["missing folder"]);
      
      		    // Check for folder id attribute existence */
      		    if (!$this->sFID = $this->oDOMFolder->getAttribute("uid"))
      	  			    throw new Exc($aError["missing fid"]);
              
              /* Check if there is a folder with specified id */
              $this->oFolder = $this->oAccount->getFolderWithAutoCreate($this->sFID);
     	  	  
     	  	    //Item read only for CSV action
     	  	    if ($this->sAction=='csv'){
                foreach($this->oDOMDoc->query('/iq/import:query/import:account/import:folder/import:item') as $item)
      	  		     $this->oDOMItems[] = $item;     
      
                //Check single or multiple item(s) get  
          	  	if (isset($this->oDOMItems[0]) && $oDOMItem = $this->oDOMItems[0])
          	  	{
          	  	    if ($this->sIID = $oDOMItem->getAttribute("uid"))
          	  	    {
          	  	          $this->bGetSingleItem = true;
          	  	          if ($this->sAttId = $oDOMItem->getAttribute("atid")) 
                            $this->bGetAttach = true;
                          
                          
          	  	          if (!is_object($this->oFolder->getItem($this->sIID,NO_ADDONS)))
          	  	                throw new Exc($aError['i_item_existence'],'i_item_existence');
          	  	    } 
          	  	   
        		    }           
                
                /* Check if there is an item element */
      					if (!$this->oDOMItem = $this->oDOMItems[0])
      						  throw new Exc($aError["i_missing_item"],'i_missing_item');	
      	  			    
      					/* Check if there is an item values specified*/
      					if (!$this->oDOMValues = $this->oDOMDoc->getNode("import:values",$this->oDOMItem)) 
                    throw new Exc($aError['missing values']);
              }
				    break;
          }
				} else{
				  if (!$this->sLines = $this->oDOMDoc->getNodeValue("import:lines",$this->oDOMImport))
					  throw new Exc("missing_lines_tag","missing_lines_tag");         
        }
        						
		}

	  /**
		* @brief	Private: Do operations dependent on 'query xmlns'
		* @return		XML subanswer (string)
		*/
		private function exeInputXML()
		{		
        $oUser = &$_SESSION['user'];
        
        $aInfo = Tools::parseFullPath($this->sFullPath,'file');

				$aAttachments = $oUser->getAttachments($aInfo['folder']);
				$aAttachment = $aAttachments[$aInfo['item']];							
        
        switch($this->aAttrs["type"])
				{
					case 'get': 
            $aCSV = $this->csv2array($aAttachment['file'],$this->sLines);
            
            $this->sTemplateFile= 'webmailiqimport_get';
            $aCSV = $this->cnvCSV($aCSV);	
            $this->aData['col'] = $aCSV;
          break;		
					case 'set':
					 switch($this->sAction){
					 
           //Import adress book from CVS
           case 'csv':
            $aCSV = $this->csv2array($aAttachment['file'],$this->sLines);
            
            $aArray = Tools::makeTreeFromXML($this->oDOMItem);
            
            if($this->sSkipFirst) unset($aCSV[0]);
            foreach($aCSV as $contact){
            $aContactArray = $aItem = array();
            $aContactArray = $this->fillArray($aArray,$contact);
            
            foreach($aContactArray['@childnodes']['values'][0]['@childnodes'] as $var => $val)
              $aItem[$var] = $val[0]['@value'];
            
            $this->oFolder->createItem($aItem,$aContactArray);
            }
           break;
           case 'groupware':
            $sGroupContentXML = file_get_contents($aAttachment['file']);
            
            if(!$oGWAccount = $this->oAccount->gwAccount)
              throw new Exc('import_invalid_account','export_invalid_account');
            
            $sGWSessionID = $oGWAccount->gwAPI->OpenGroup('*');
            $oGWAccount->gwAPI->FunctionCall("ImportGroup",$sGWSessionID,$sGroupContentXML);

           break;
           case 'vcard':
           case 'vcalendar':
           case 'ldif':           
            $sContent = file_get_contents($aAttachment['file']);
            $this->oFolder->importItem($this->sAction,$sContent);
           break;
          }
          break;
				}

				 
		}
		
		private function csv2array($filename,$lines = false,$separator = ","){
      $fp = fopen ($filename,"r");
      while ($data = fgetcsv ($fp, 1000, $separator)) {
        $aResult[] = $data;
        if($lines && ++$lCount>=$lines) break;
      }
      fclose ($fp);     
      return $aResult;
    }
    
    private function cnvCSV($aCSV){
      foreach($aCSV as $sLineNumber => $aLine)
        foreach($aLine as $sColNumber => $sColValue){
          $aResult['num'][$sColNumber]['line']['num'][$sLineNumber]['value'] = $sColValue;
          $aResult['num'][$sColNumber]['line']['num'][$sLineNumber]['id'] = $sLineNumber;
          $aResult['num'][$sColNumber]['id'] = $sColNumber;
        }
      return $aResult;
    }
    
    private function fillArray($aArray,$aCSVLine){
      $aArray = $this->fill_r($aArray,$aCSVLine);
      return $aArray;
    }

    private function fill_r($aArray,$aCSVLine){
      foreach($aArray as $key => $val){
        if (is_array($val)) $aResult[$key] = $this->fill_r($aArray[$key],$aCSVLine);
        else{ 
          if($aCSVLine[$val]){ 
            $aResult[$key] = $aCSVLine[$val];
          }
          else{
           $aResult[$key] = $val;
          }
        }  
      }
      return $aResult;
    }
}
		
?>
