<?php


require_once 'error.php';
require_once 'mailparse.php';
require_once 'webmailiqfolders.php';
require_once 'webmailiqmessage.php';
require_once 'mail.php';

/**
 * @brief class for handling webmailIqItems namespace
 * @author Pavel Perna
 * @revision 06.11.2006
 */
 class WebmailIqItems extends XMLRequestInterface {
 
		// Array with allowed actions
		private $aActions = array('edit'=>1, 'delete'=>1, 'add'=>1,'attachments'=>1,'move'=>1, 'copy'=>1,'blacklist'=>1,'whitelist'=>1,'deliver'=>1,'redirect'=>1,'search'=>1);
	 
		/**
		* @brief Public: Constructor
		* @param $oQuery A DOMElement filled with query element
		* @param $oDoc	A DOMDocument of the request
		* @param $attrs  An Array with parent attributes
		*/
		public function __construct($oDOMQuery,&$oDOMDoc,&$attrs)
		{

				$this->oDOMQuery = $oDOMQuery;
				$this->oDOMDoc = &$oDOMDoc;
				$this->aAttrs = &$attrs;
				
				//Get all or only one item(s)?
				$this->bGetSingleItem = false;
				
				$this->checkInputXML();
				$this->exeInputXML();

		}  
		
	  /**
		* @brief	Private - Checks if the input XML is valid
		* @return  TRUE/FALSE and throws exception
		*/		
		private function checkInputXML()
		{
		  global $aError;
	  	  
		    // Check if there is an account element in query 
	  	  if (!$this->oDOMAccount = $this->oDOMDoc->getNode("items:account",
                                                          $this->oDOMQuery))
              throw new Exc($aError["missing account"]);
	  	  
	  	  // Check if an account id is set 
	  		if (!$this->sAID = $this->oDOMAccount->getAttribute("uid"))
	  			    throw new Exc($aError["missing aid"]);
	  			
	  		// Check if there is a account with specified id 
	  		if (!$this->oAccount = $_SESSION['user']->getAccount($this->sAID))
	  			    throw new Exc($aError["invalid aid"]);

	  		// Check for folder element existence 
	  		if (!$this->oDOMFolder = $this->oDOMDoc->getNode("items:folder",
                                                         $this->oDOMAccount))
	  			    throw new Exc($aError["missing folder"]);

		    // Check for folder id attribute existence */
		    if (!$this->sFID = $this->oDOMFolder->getAttribute("uid"))
	  			    throw new Exc($aError["missing fid"]);
        
        /* Check if there is a folder with specified id */
        $this->oFolder = $this->oAccount->getFolderWithAutoCreate($this->sFID);

        if (!$this->oFolder 
            && $this->oFolder->type!='M' 
            && $this->oAccount->gwAccount->bLogged == false)
          throw new Exc('Groupware initialization failed','gw_init_error');
          
        //Store unsubscribed folders 1st synchronization till next account->refresh/sync
        if (!$this->oFolder->gw 
        && !$this->oFolder->sync 
        && !$this->oAccount->aSyncedFolders[$this->oFolder->folderID])
        {
          $this->oAccount->aSyncedFolders[$this->oFolder->folderID] = true;
          $this->oFolder->sync();
        }

        //Parse items
	  	  $this->oDOMItems = array();	
	  	  $this->sAttId = false;
 	  	  foreach($this->oDOMDoc->query('/iq/items:query/items:account/items:folder/items:item') as $item)
	  		     $this->oDOMItems[] = $item;     

        //Check single or multiple item(s) get  
  	  	if (isset($this->oDOMItems[0]) && $oDOMItem = $this->oDOMItems[0])
  	  	{
  	  	    if ($this->sIID = $oDOMItem->getAttribute("uid"))
  	  	    {
  	  	          $this->bGetSingleItem = true;
  	  	          if ($this->sAttId = $oDOMItem->getAttribute("atid")) 
                    $this->bGetAttach = true;
                  
                  
  	  	          if (!is_object($this->oFolder->getItem($this->sIID,NO_ADDONS)))
  	  	                throw new Exc($aError['i_item_existence'],'i_item_existence');
  	  	    } 
  	  	   
		    } 
	
	  		/* TODO : SET items */
	  		if ($this->aAttrs['type']=='set'){


					/* Check if there is an item element */
					if (!$this->oDOMItem = $this->oDOMItems[0])
						  throw new Exc($aError["i_missing_item"],'i_missing_item');	
						  
					/* Check if there is an item action specified*/
					if (!$this->sIAction = $this->oDOMItem->getAttribute("action"))
						  throw new Exc($aError["i_missing_action"],'i_missing_action');	

					/* Check if actual action is correct*/
					if (!isset($this->aActions[$this->sIAction]))
						  throw new Exc($aError["i_invalid_item"],'i_invalid_item');  

        //Do some check before specific actions
				switch($this->sIAction)
				{	 
					  case 'add':
            case 'edit':
					  case 'delete':
					 		  /* Check if there is an item values specified*/
						if (!$this->oDOMValues = $this->oDOMDoc->getNode("items:values",$this->oDOMItem)) 
						{
								  //LogError::Error($aError['missing values']);
								  if (!($this->sIAction=='delete'))
                    throw new Exc($aError['missing values']);
						}

						if ($pom = $this->oDOMDoc->getNode("items:folder",$this->oDOMValues))
						  $this->sSpamFolder = $pom;

						break;
						case 'edit':
					  case 'delete':
					  case 'move':
					  case 'copy':
					  case 'attachments':
							if (!$this->sIID = $this->oDOMItem->getAttribute("uid"))
								  throw new Exc($aError['missing id']);
					  break;
        }
        //POP3Class FIX  
        if ($this->sIAction=='move' && $this->oFolder->itemClassName=="POP3Item")
          $this->sIAction = 'delete';      
       			   
		  }
	  }
	  
	  /**
		* @brief	Private: Do operations dependent on 'query xmlns'
		* @return		XML subanswer (string)
		*/
		private function exeInputXML()
		{
			 global $aError;
			 // GET/SET  
			 switch ($this->aAttrs['type']) {
				 case 'get':
            $t = time();
            $m = microtime();
            
						$this->sTemplateFile = 'webmailiqitems_get';
            $aFilterTag = $this->filterAndTags();

            if($sPhrase = $aFilterTag['fulltext'])
            {
              $aItems = $this->oFolder->search($sPhrase,$aFilterTag);

            } else {

              //Handle single item get vs. multiple item get
              if ($this->bGetSingleItem)
              {
      
                    $oItem = $this->oFolder->getItem($this->sIID);
                    if ($oItem->wmclass=="M")
                     $oItem->markAsRead();
                    //Attachments
                    if ($this->bGetAttach)
                    {
      
                      $attachment = new GroupWareAddon($oItem,'attachment');
                      $value = $attachment->getAttachment($this->sAttId);
                      $value = explode(':',$value);
                      $value = $attachment->getGroupAttachment($value[0],$value[1],$value[2]);
                      
                      $this->sAID = $value['account'];
                      $this->sFID = $value['folder'];
                      $this->sIID = $value['item'];
                      
                      $oAccount = $_SESSION['user']->getAccount($this->sAID);
                      $oFolder = $oAccount->getFolder($this->sFID);
                      $oItem = $oFolder->getItem($this->sIID); 
      
                    }
                    $aItems = array(0 => $oItem);
      
              } else {
      
                    $aItems = $this->oFolder->getItems($aFilterTag);
              }
            }

            //Prepare data for template
            $this->aData['items']['num'] = $this->cnvItems($aItems,$aFilterTag,$this->bGetSingleItem); 

						$this->aData["values"] = !(isset($this->oFolder->gw) && $this->oFolder->gw);
						$this->aData['aid'] = htmlspecialchars($this->sAID);
            $this->aData['fid'] = htmlspecialchars($this->sFID);

            $this->aData['counter'] = $this->bGetSingleItem?1:$this->oFolder->countItems(0,true,$aFilterTag['sql']);
          
            // Process Mail folder 
				    if (!isset($this->oFolder->gw) || !$this->oFolder->gw)     
      					$this->aData['recent']  = $this->oFolder->countItems(Item::FLAG_SEEN, false);           
            // Process GW folder 
             else 
      					$this->aData['recent']  = '0'; 
            
            $offset = intval($aFilterTag['offset']);
            if ($this->aData['counter']>0
                && $offset<$this->aData['counter']
                && count($this->aData['items']['num']) == 0)
            $this->aData['counter'] = 0; 
            
            $time = time()-$t+microtime()-$m;
            log_buffer("WebmailIqItems: Get (time:".$time.")",'DEBUG');  
            
            break;
				 case 'set':

				    //Create instances of Cache, API and optionaly IMAP objects
				    //to speed up transactions with many items
						$cache['cache'] = new Cache($this->oAccount->user,$this->oAccount->protocol);
	          //$cache['cache']->transaction();
						$cache['api'] = new MerakAPI;	

            if (strtolower(get_class($this->oFolder))=='imapfolder') {
            	$account = $this->oAccount;
              $cache['imap'] = IMAP::instance($account);
            } else {
              $cache['imap'] = false;
            }            

            
            //Do actions			    
						switch ($this->sIAction)
						{
                //Delete an item
                case 'delete':

                $tp = time();
                $mp = microtime();
                
                foreach($this->oDOMItems as $oDOMItem){
							    $sUID = $oDOMItem->getAttribute('uid');
							    // Groupware 
                  // Check wheter we delete all events in the serie or the single one
								  if ($oDOMExpdate = $this->oDOMDoc->getNode("items:expdate",$this->oDOMValues)){
								    $sExpdate = $oDOMExpdate->nodeValue;
                    //Handle additional data for reccuring events

                    if ($this->oDOMFollowing = $this->oDOMDoc->getNode("items:expfollowing",$this->oDOMValues))
                      $bFollowing = $this->oDOMFollowing->nodeValue=='true'?true:false;
										else  
                      $bFollowing = false;

                    //Delete GW item on specified day
                    $oItem  = $this->oFolder->getItem($sUID);
                    $oItem->delete($sExpdate,$bFollowing);
                  
                  //E - mails
                  //Delete emails or non-repeating gw items
                  } else{
                    $oItem = $this->oFolder->getItem($sUID,$cache);
                    $aItemsToDelete[] = $oItem;
                  }
                }
                
                $timeprep = time() - $tp + microtime() - $mp;
                log_buffer("DeleteItems: Prepare (time:".$timeprep.")",'DEBUG');
                
                $tdel = time();
                $mdel = microtime();
                                
                //Multiple item delete - optiomalization
                if ($aItemsToDelete)
                    $this->oFolder->deleteItems($aItemsToDelete);
                
                $timedel = time() - $tdel + microtime() - $mdel; 
                log_buffer("DeleteItems: Delete (time:".$timedel.")",'DEBUG');
                
							  break;
							  //Add new item
							  case 'add':

                  if ($this->oFolder->gw || $this->oFolder->type=='Q' || $this->oFolder->type=='QL') 
                  {  
                    
                  //$param = $this;
									//Cannot create messages using IQ:Items ( use IQ:Message)
                  }else 
                    throw new Exc($aError['i_create'],'i_create');
                    	
                  $aItem = Tools::makeArrayFromXML($this->oDOMValues);
                  $aTreeItem = Tools::makeTreeFromXML($this->oDOMItem);
                  
									$this->oFolder->createItem($aItem,$aTreeItem);								  									 
							  break;
								//Edit existing item
								case 'edit':

								 foreach($this->oDOMItems as $oDOMItem)
								
								  //Email
								  if (!isset($this->oFolder->gw) || !$this->oFolder->gw) {
										//EMail -  it is allowed to edit message flags and color only
										if ($oFlag = $this->oDOMDoc->getNode("items:values/items:flags", $oDOMItem)) {
											$sFlags = $oFlag->nodeValue;
											$oItem = $this->oFolder->getItem($oDOMItem->getAttribute('uid'));
											$oItem->setFullFlags($sFlags);
										
										} else if ($oColor = $this->oDOMDoc->getNode("items:values/items:color", $oDOMItem)) {
                      $sColor = $oColor->nodeValue;
											$oItem = $this->oFolder->getItem($oDOMItem->getAttribute('uid'));
											$oItem->changeColor($sColor);
										} else
											throw new Exc($aError['i_edit'],'i_edit');
									
                  //GroupWare					//GW items
									}	else {

										//$aFilterTag = $this->filterAndTags();
										$this->oDOMValues = $this->oDOMDoc->getNode("items:values",$oDOMItem);
								    $oItem = $this->oFolder->getItem($oDOMItem->getAttribute('uid'),NO_ADDONS);

                    //Handle additional data for reccuring events
                    if ($this->oDOMDate = $this->oDOMDoc->getNode("items:expdate",$this->oDOMValues))
                      $sExpdate = $this->oDOMDate->nodeValue;
										else  
                      $sExpdate = false;
                      
                    if ($this->oDOMFollowing = $this->oDOMDoc->getNode("items:expfollowing",$this->oDOMValues))
                      $bFollowing = $this->oDOMFollowing->nodeValue=='true'?true:false;
										else  
                      $bFollowing = false;
                    
                    $aItem = Tools::makeArrayFromXML($this->oDOMValues,true);
                    $aTreeItem = Tools::makeTreeFromXML($this->oDOMItem);
                                      
                    $oItem->edit($aItem,$aTreeItem,$sExpdate,$bFollowing);
                  }
                  
							  break;
								//Copy & move
								case 'move':
								case 'copy': 
								foreach($this->oDOMItems as $oDOMItem)
								{
								  $sUID = $oDOMItem->getAttribute('uid');
								if (!$sFolderName = $this->oDOMDoc->getNodeValue("items:folder",$oDOMItem))
									throw new Exc($aError['i_nodestination'], 'i_nodestination');

								// TODO: support moving to different account
									$sAccountName = $this->oDOMDoc->getNodeValue("items:account",$oDOMItem);
                  
                  if ($this->oFolder->gw)
                    $sFolderName = str_replace("/","\\",$sFolderName);
								  
                // compare types of folder
                
      					$destination = $this->oAccount->getFolderWithAutoCreate($sFolderName);								

								if ($this->oFolder->type != $destination->type)
									throw new Exc($aError['i_baddestinationtype'], 'i_baddestinationtype');
									

								$oItem = $this->oFolder->getItem($sUID,$cache);									
								
								$aItemsToMove[] = $oItem;
								}

                if ($aItemsToMove){
                  if ($this->sIAction == 'move')
									 $this->oFolder->moveItems($destination,$aItemsToMove, $cache);
									else{
                   $this->oFolder->copyItems($destination,$aItemsToMove, $cache);
                  }
                }

                break;
						    // Quarantime actions
						    case 'deliver':
						      if ($this->oFolder->type=='Q')
						        foreach($this->oDOMItems as $oDOMItem)
									   $this->oFolder->deliverItem($oDOMItem->getAttribute('uid'));
                break;
                
                case 'whitelist':
						      if ($this->oFolder->type=='Q' 
                  ||  $this->oFolder->type=='QL')
						        foreach($this->oDOMItems as $oDOMItem){
						        
									   if ($this->oFolder->type=='Q') 
                      $deliver = true;
                     else 
                      $deliver = false;
                     $this->oFolder->whitelistItem($oDOMItem->getAttribute('uid'),false,$deliver);
                     
                     
                    }
                break;
							  case 'blacklist':
						      if ($this->oFolder->type=='Q' 
                  ||  $this->oFolder->type=='QL')
						        foreach($this->oDOMItems as $oDOMItem)
									   $this->oFolder->blacklistItem($oDOMItem->getAttribute('uid'));							     
                break;
                case 'redirect':
                  foreach($this->oDOMItems as $oDOMItem)
									{
									 $oItem = $this->oFolder->getItem($oDOMItem->getAttribute('uid'));
									 $message = $oItem->getMessage();

									 //No CRLF at the end of message
                   if (substr($message,strlen($message)-2,2)!=CRLF)
                    $message.=CRLF.CRLF;
									 
                   $message = explode(CRLF.CRLF,$message,2);
                   
                   $mail = new Mail();
                   $mail->SetFrom($_SESSION['EMAIL']);
                   
                   foreach($this->oDOMDoc->query('items:to',$oDOMItem) as $to)                   
                    $mail->AddTo($to->nodeValue);
                   
                   foreach($this->oDOMDoc->query('items:distrib',$oDOMItem) as $distrib)
                      foreach($this->oDOMDoc->query('items:account',$distrib) as $account)
                        foreach($this->oDOMDoc->query('items:folder',$account) as $folder)
                          WebmailIQMessage::processDistributedList('items',$_SESSION['user'],$this->oDOMDoc,$account->getAttribute('uid'),$folder->getAttribute('uid'),$folder,$mail);
                   
                   $mail->Send($message[0],$message[1]);
                  }
                break;
                case 'search':
                  $sPhrase = $this->oDOMDoc->getNodeValue("items:fultext",$this->oDOMFolder);
                  $oFolder = &$oAccount->getFolder($this->sFID);
                  $oFolder->search($sPhrase);
                break;
								default:
									
								break;
						
						}
						break;		  
			 }
		}
		
	  /**
		*  @brief Checks and returns filter and selected tags
				  and returns them as array
			@return Filter and Tag array
		*/			
		private function filterAndTags(){  
    global $oDOMItem,$oDOMDoc;
    
    $oDOMDoc = $this->oDOMDoc;
		// init the $aFilterTags array
		$aFilterTags = array();
		$aFilterTags['offset'] = 0;
		$aFilterTags['limit'] = '';
		$aFilterTags['sql'] = '';
		$aFilterTags['interval'] = '';
		$aFilterTags['tag'] = '';
		$aFilterTags['orderby'] = '';
		$aFilterTags['gwAddons'] = array();

			/* Check if there is an item and filter element */
		 if ($this->oDOMItem = $this->oDOMDoc->getNode('items:item',$this->oDOMFolder))
		 {		
				  //Filter
		  		  if ($oDOMFilter = $this->oDOMDoc->getNode('items:filter',$this->oDOMItem)){
						//Limit?
						if ($oDOMLimit = $this->oDOMDoc->getNode("items:limit",$oDOMFilter))
							$aFilterTags['limit'] = $this->oDOMDoc->getNodeValue('items:limit',$oDOMFilter);
						
            if ($oDOMLimit = $this->oDOMDoc->getNode("items:offset",$oDOMFilter))
							$aFilterTags['offset'] = $this->oDOMDoc->getNodeValue('items:offset',$oDOMFilter);
						
            if ($oDOMOrderBy = $this->oDOMDoc->getNode("items:order_by",$oDOMFilter))
							$aFilterTags['orderby'] = $this->oDOMDoc->getNodeValue('items:order_by',$oDOMFilter);

            //FIXME : security chceck ? 
						if ($oSQL = $this->oDOMDoc->getNode('items:sql',$oDOMFilter))
							$aFilterTags['sql'] = $this->oDOMDoc->getNodeValue("items:sql",$oDOMFilter);

						if ($oDOMInterval = $this->oDOMDoc->getNode('items:interval',$oDOMFilter))
							$aFilterTags['interval'] = $this->oDOMDoc->getNodeValue("items:interval",$oDOMFilter);
							
						if ($oDOMFulltext = $this->oDOMDoc->getNode('items:fulltext',$oDOMFilter))
							$aFilterTags['fulltext'] = $this->oDOMDoc->getNodeValue("items:fulltext",$oDOMFilter);
							
				  }
				  
				  //If item id is specified , return only this item
				  if ($sIID = $this->oDOMItem->getAttribute('uid')) {	 
						if (strlen($aFilterTags['sql']))
							$aFilterTags['sql'] = '(' . $aFilterTags['sql'] . ') AND ';
						$aFilterTags['sql'] .= "item_id='".$sIID."'";
					}
				  
				  //Tags
				  if($oDOMValues = $this->oDOMDoc->getNode('items:values',$this->oDOMItem)){
				    $tagvalues = Tools::makeArrayFromXML($oDOMValues,true);
				    $tags = @array_keys(Tools::makeArrayFromXML($oDOMValues,true));
				    foreach ($tags as $key=>$tag){
						      // rename some tags - their value is different in the protocol than in the server
									switch (strtolower($tag)) {
										case 'from':
											$tags[$key] = 'header_from';
											break;
										
										case 'to':
											$tags[$key] = 'header_to';
											break;
                    case 'ctz':
                      unset($tags[$key]);
                      $aFilterTags['timezone'] = $tagvalues[strtolower($tag)];
                    break;
                   // case 'all_headers':
                   //   $tags[$key] = '*';
                  //  break;
                    default:
											// do nothing
											break;
									}
             }
            //GW FIX
            $aFilterTags['tag']= @implode(',',$tags);
          }
          //XML parser trouble fix
          if ($aFilterTags['tag']=="#text") $aFilterTags['tag']=false;

				  if (isset($this->oFolder->gw) && $this->oFolder->gw){
						$aAddons = array('recurrence'=>'recurrence',
										 'reminders'=>'reminder',
										 'contacts'=>'contact',
										 'note'=>'note',
										 'locations'=>'location');
						foreach ($aAddons as $key => $addon)
							  if($subnode = $this->oDOMDoc->getNode("items:".$key,$this->oDOMItem))
								  $aFilterTags['gwAddons'][] = $addon;
								  
						//If item id is specified , return only this item
						if ($sIID)
						{
							  
								$sPrefix = isset($this->type) && $this->type=='C' ? 'ITM' : 'EVN';
								$aFilterTags['sql'] = $sPrefix."_ID='".$sIID."'";
						}
				  }
				 $oDOMItem = $this->oDOMItem; 
		 }	

		 return $aFilterTags;			  
		}	
		
	  /**
		*  @brief Converts folder array into template compatible data array
		*  @param $folders Array of folder objects
		*  @return folders array in the form for template system
		*/			
		public static function cnvItems($items,$aFilterTag = array(),$bSingle = false){
      global $_iqdata;
			if (!is_array($items)) 
				return;

			foreach($items as $oItem)
			{
				$aItem = array();
				$aItem["id"] = $oItem->itemID;

			  switch($oItem->wmclass)
			  {
           //Mail			     
           case 'M':
			       $aItem = self::cnvMailItem($oItem,$aFilterTag);
			       $aItem["mail"] = 1;
           break;
			     
			     //GroupWare
			     case 'GW':
			     	/*Fixme : do not store sFields in GroupWareItem, call XML
					    transformation in this method */
					   $aItem["sFields"] = $oItem->sFields;
			     break;
			     
           //Quarantine
			     case 'Q':

              $aItem["owner"] = (string) $oItem->owner;
              $aItem["sender"] = (string) $oItem->sender;
               
              $oldAFilter = $aFilterTag; 
              $array = explode(",",$aFilterTag['tag']);
			        foreach($array as $item)
				        $array2[trim($item)] = 1;
			
			        $aFilterTag['tag'] = $array2;
              
               
               if($aFilterTag['tag']['header_to'])  
                $aItem["to"] = $aItem["owner"];

  	           if($aFilterTag['tag']['header_from']) 
                $aItem["from"] = $aItem["sender"];

               $aItem["subject"] = (string) $oItem->subject;
                
  	           $aItem["domain"] = (string) $oItem->domain;
  	           $aItem["createdat"] = (string) $oItem->createdAt;
  	           $aItem["createdon"] = (string) $oItem->createdOn;
  	           $value = jdtounix($oItem->createdOn) + $oItem->createdAt;
  	           $aItem["date"] = (string) $value;

             //if ($_iqdata['format']=='xml')
                	           
  	           
             if ($bSingle && $oItem->folder->type=='Q') {
              $aItem = array_merge(self::cnvMailItem($oItem,$oldAFilter),$aItem);

              $aItem["mail"] = 1;
              $aItem["quarantine"] = 0;

             }else{ 
               $aItem = Tools::htmlspecialchars_array($aItem);  
  	           $aItem["quarantine"] = 1;
  	           $aItem["mail"] = 0;
  			     }  


  	           //$aItem["time"] = $oItem->createdAt;              
         

           break;			     
			     
           //TODO: RSS items
			     case 'RSS':
			     break;
        }

				$aItems[] = $aItem;
				unset($aItem);
			}

			return $aItems;
		}
		
 /*   private static function tagStr2Arr($string)
		{
			//Get list of tags	  
			$array = explode(",",$aFilterTag['tag']);
			foreach($array as $item)
				$array2[trim($item)] = 1;
			
			$aFilterTag['tag'] = $array2;
			return $aFilter
    }*/
		
		private static function cnvMailItem($oItem,$aFilterTag = array())
		{
      global $_iqdata;

    // Default message tags 
			if ($aFilterTag['tag']=='*') 
				$aFilterTag['tag'] = 'item_id, rid,	size,	date,	flags, priority, static_flags,
															header_from, header_to,	subject, html, text, color, bcc, 
															cc, smime_status, has_attachment, attachments, confirm_addr';
       
			$array = explode(",",$aFilterTag['tag']);
			foreach($array as $item)
				$array2[trim($item)] = 1;
			
			$aFilterTag['tag'] = $array2;
			
			$aItems = array();		  
			
      //Do we have to parse a message?
			if (isset($aFilterTag['tag']['html'])
				||  isset($aFilterTag['tag']['attachments'])
				||  isset($aFilterTag['tag']['text'])
				||  isset($aFilterTag['tag']['cc'])
				||  isset($aFilterTag['tag']['bcc'])
				||  isset($aFilterTag['tag']['confirm_addr'])
        ||  isset($aFilterTag['tag']['all_headers']))
			{

				$message = $oItem->parseMessage();
				
				
        if($_SESSION['FULLTEXT'] && $message['plain_body'][0])
          $oItem->updateItem(array('body'=>$message['plain_body'][0]));
        
        if (isset($aFilterTag['tag']['reply_to'])) $aItem["reply_to"] = (string) ($message['headers']['reply-to']);
  		
	      if (isset($aFilterTag['tag']['all_headers'])) $aItem["all_headers"] = $message['headers_plain'];

				//TEXT body
				if (isset($aFilterTag['tag']['text']) && $aFilterTag['tag']['text']) {
					if (isset($message['plain_body'][0]))
						$aItem["text"] = rtrim($message['plain_body'][0]);
					else
						$aItem["text"] = '';
				} 

				//HTML body
				if (isset($aFilterTag['tag']['html']) && $aFilterTag['tag']['html']) {
					if ($message['html_body'][0]){
        		$value = Tools::cleanHTML($message['html_body'][0],$base);
        	  if ($base)
        	   $aItem["base"] = $base;
          } else {
            if ($message['plain_body'][0]){
				      //TODO use function to make hrefs from text links
				      $value = nl2br(Tools::textLinksToHTML(htmlspecialchars(trim($message['plain_body'][0]))));

             } else {
						  $value = '';
             }
          }

          //Try if HTML can be read by XML parser and try to fix it if not
					$pom = "<test>".htmlspecialchars($value)."</test>";
          $xml = @simplexml_load_string($pom);
          if (!is_object($xml)){
            $value = iconv("UTF-8","UTF-8//IGNORE",$value);
          }
          $aItem["html"] =  $value;
        }

				//Attachments
				if (isset($aFilterTag['tag']['attachments']) && $aFilterTag['tag']['attachments'] && $message['attachments']) { 
					foreach($message['attachments'] as $attachment){
						$aItem['attachments']['num'][] = $attachment;
						
					}
				}
				//Carbon copies
				if (isset($aFilterTag['tag']['cc']) && $aFilterTag['tag']['cc'] && $message['headers']['cc']) 
					$aItem['cc'] = (string) ($message['headers']['cc']);
				//Blind carbon copies
				if (isset($aFilterTag['tag']['bcc']) && $aFilterTag['tag']['bcc'] && $message['headers']['bcc']) 
					$aItem['bcc'] = (string) ($message['headers']['bcc']);
				//X-Confirm-Reading-To
				if (isset($aFilterTag['tag']['confirm_addr']) && $aFilterTag['tag']['confirm_addr'] && $message['headers']['custom-headers']['x-confirm-reading-to']) 
					$aItem['confirm_addr'] = (string) ($message['headers']['custom-headers']['x-confirm-reading-to']);
			}


        if (isset($aFilterTag['tag']['static_flags'])) {$aItem["static_flags"] =  (string) $oItem->staticFlags;$aItem["has_st_flags"] = true;}
  			if (isset($aFilterTag['tag']['smime_status'])) {$aItem["smime_status"] =  (string) $oItem->sMimeStatus;$aItem["has_smime"] = true;}
  			if (isset($aFilterTag['tag']['color'])) {$aItem["color"] =  (string) $oItem->color;$aItem["has_color"] = true;}
  			if (isset($aFilterTag['tag']['has_attachment'])) {$aItem["has_attachment"] =  $oItem->hasAttachments ? 'true' : 'false';}
        if (isset($aFilterTag['tag']['flags'])) {$aItem["flags"] =  (string) $oItem->flags;$aItem["has_flags"] = true;}
        if (isset($aFilterTag['tag']['size'])) $aItem["size"] = (string) $oItem->size;
        if (isset($aFilterTag['tag']['reply_to'])) $aItem["reply_to"] = (string) ($message['headers']['reply-to']);
  			if (isset($aFilterTag['tag']['priority'])) $aItem["priority"] = (string) $oItem->priority;
  			if (isset($aFilterTag['tag']['date'])) $aItem["date"] = (string) $oItem->date;
        if (isset($aFilterTag['tag']['subject'])) {$aItem["subject"] = (string) ($oItem->subject); $aItem["has_subject"] = true;}
  	    if (isset($aFilterTag['tag']['header_from'])) $aItem["from"] = (string) ($oItem->from);
        if (isset($aFilterTag['tag']['header_to'])) if ($oItem->to) $aItem["to"] = (string) ($oItem->to);
        
		  
	    $aItem['id']=$oItem->itemID;

			$aItem = Tools::htmlspecialchars_array($aItem);

			
			return $aItem;
		}

}

?>
