#!/bin/sh
#
# Merak Mail Server Installation package
# (c) IceWarp Ltd. 1999-2006
#
#	All Rights Reserved
#
# http://www.icewarp.com 
#
#

# defaults

USE_COLORS="0"

NEWVERSION="9.1.0"

MERAKDIR="/opt/merak"
USER="root"
GROUP="root"
CHROOT="0"
MERAKCONF="${MERAKDIR}/config/merak.conf"
ERRORLOG=~/merak-install-error.log
INSTALLLOG=~/merak-install.log

# init

SELF=$0
WORKDIR=`dirname $0`

if  [ $TERM = "linux" ] || [ $TERM = "xterm" ] || [ $TERM = "xterm-color" ] || [ $TERM = "screen" ] && [ "$USE_COLORS" = "1" ]
	then
		GOOD=$'\e[32;01m'
		WARN=$'\e[33;01m'
		BAD=$'\e[31;01m'
		NORMAL=$'\e[0m'
		HILITE=$'\e[37;01m'
		BRACKET=$'\e[34;01m'
	else
		GOOD=$'\e[0;32m'
		BAD=$'\e[0;31m'
		NORMAL=$'\e[0m'
		WARN=${NORMAL}
		HILITE=${NORMAL}
		BRACKET=${NORMAL}
fi

preconf()
	{
		if [ -f $MERAKCONF ]
			then
				source $MERAKCONF
				[[ $InstallDir != "" ]] && MERAKDIR=$InstallDir
				[[ $User != "" ]] && USER=$User
				[[ $Group != "" ]] && GROUP=$Group
				[[ $Chroot != "" ]] && CHROOT=$Chroot
				# hack to fill in version number if empty
				OLDVERSION=$Version
				export UPGRADE="1"	
		fi
	}


good()
	{
		echo "${GOOD}**	${NORMAL}$*"
	}


bad()
	{
		echo "${BAD}**	${NORMAL}$*"
	}

warn()
	{
		echo "${WARN}**	${NORMAL}$*"
	}

hilite()
	{
		echo "${GOOD}**	${HILITE}$*${NORMAL}"
	}

getparam()
	{
		echo -n "${GOOD}**	${NORMAL}$1 [$2]${NORMAL}: "
		read PARAM
	}


extract_image()
	{
		good "Please wait..."
		if [ "$UPGRADE" = "1" ]
			then
				tar -xf $1 -C $2
			else
				tar -xf $1 -C $2
				cp -f $2/config/default/* $2/config/
				cp -f $2/webmail/config/default/* $2/webmail/config/
				cp -f $2/spam/rules/default/local.cf $2/spam/rules/local.cf
		fi
		if [ -e info.xml ]
			then
				mkdir -p $2/xml 2>/dev/null  
				cp -f info.xml $2/xml/
		fi
	}

ask_questions()
	{
		getparam "Installation prefix" "$MERAKDIR"
		[[ $PARAM != "" ]] && MERAKDIR=$PARAM
		
		# repeat here in case we are upgrading
		MERAKCONF="$MERAKDIR/config/merak.conf"		
		preconf
		
		if  [ "$UPGRADE" = "1" ]
			then
			    echo "${GOOD}** Performing upgrade"
			else
				getparam "Run services under user" "$USER"
				[[ $PARAM != "" ]] && USER=$PARAM
		
				getparam "Run services under group" "$GROUP"
				[[ $PARAM != "" ]] && GROUP=$PARAM

				#getparam "Run services in chroot" "$CHROOT"
				#[[ $PARAM != "" ]] && CHROOT=$PARAM
		fi
	}

confirm_params()
	{
		warn ""
		warn "Please check the following before continuing:"
		warn ""
		if [ -d $MERAKDIR ]
			then
				good "Installation prefix:	$MERAKDIR (path exists, upgrading)"
			else
				good "Installation prefix:	$MERAKDIR (will be created, installing)"
		fi
		
		if ( grep -q "^$USER:" /etc/passwd )
			then
				good "Services will run under user:	$USER (user exists)"
			else
				bad "User $USER doesn't exist!"
		fi
	
		if ( grep -q "^$GROUP:" /etc/group )
			then
				good "Services will run under group:	$GROUP (group exists)"
			else
				bad "Group $GROUP doesn't exist!"
		fi
		
		if ( id $USER 2>&1 |grep -q "($GROUP)" )
			then
				good "	User $USER is a member of group $GROUP"
			else
				warn "	User $USER is not a member of group $GROUP!"
		fi
		if [[ $CHROOT == "1" ]]
			then
				good "Services will run in chroot jail"
		fi
		
		good ""
		warn "	Press ENTER to continue, CTRL+C to quit"
		read -s
		echo ""
	}

install_package()
	{
		good "Creating $MERAKDIR directory"
		mkdir -p $MERAKDIR || bad "Failed! Check user permissions"
		good "Extracting distribution"
		extract_image "$WORKDIR/merak-image-$NEWVERSION.tar" "$MERAKDIR" || ( bad "Failed, check user permissions and available disk space. 70MB disk space is needed." ; exit 1 )
	}

configure_package()
	{
		good "Changing permissions"
		chown -R $USER:$GROUP $MERAKDIR || bad "Failed! Check user permissions on $MERAKDIR"
		if ( touch $MERAKCONF )
			then 
				cp $MERAKCONF $MERAKCONF.bak
				DATE=`date`
				echo "## Merak Mail Server configuration file
## this file was generated by IceWarp installer on $DATE
## InstallDir=/path/to/merak
## Merak installation prefix, absolute path, no useful default

InstallDir=$MERAKDIR

## Version=number
## Merak Mail Server version, for reference only

Version=$NEWVERSION
 
## User=username
## User to run services as, default=root

User=$USER
 
## Group=group
## Group to run services as, default=root

Group=$GROUP

## Capabilities
## capabilities that are set at startup, see
## man cap_from_text
## for more info
## ADVANCED USERS ONLY!

Capabilities=

## Chroot=[0/1]
## Use chroot, default=0, currently disabled

## Chroot=$CHROOT

AVENGINE=$AVENGINE

# Previous and new version number - used for upgrade purposes

OLDVERSION=$OLDVERSION" >$MERAKCONF

			else
				warn "File $MERAKCONF is not writeable, skipping postconfiguration!"
				warn "Note: You can do this later by running \"$0 fix\""
		fi
		
		if ( touch $MERAKDIR/php/php.ini )
			then
				echo "eaccelerator.cache_dir = \"${MERAKDIR}/php/tmp/\"
extension_dir = \"${MERAKDIR}/php/ext/\"
error_log = \"${MERAKDIR}/logs/php/error.log\"
open_basedir_ignore = \"${MERAKDIR}/\"
upload_tmp_dir = \"${MERAKDIR}/php/tmp/\"
session.save_path = \"${MERAKDIR}/php/tmp/\"" >>${MERAKDIR}/php/php.ini
			else
				warn "File: ${MERAKDIR}/php/php.ini is not writeable, skipping postconfiguration!"
				warn "Note: Web interface may misbehave"
		fi
	}

run_postconf()
	{
		good ""
		warn "  Press ENTER to continue, CTRL+C to quit"
		read -s
		echo ""
		$MERAKDIR/scripts/postconf
	}

copyright()
	{
		good ""
		good "Merak Mail Server installer"
		good "	© 1999-2006 IceWarp Ltd. "
		good ""
	}
	
rejoice()
	{
		good "Merak Mail Server was successfully installed or upgraded."
		good ""
		good "Installator log is available in $INSTALLLOG"
		good "Installator error log is available in $ERRORLOG"
	}

accept_license()
	{
		less +1 -1  $WORKDIR/LICENSE
		echo ""
		good "You must accept this license agreement if you want to continue."
		warn "  Press ENTER to accept license or CTRL+C to quit"
		read -s
		echo ""
	}

getscriptparams()
	{
		case $1 in
			fix)	
				copyright
				good "Configuration fixing mode entered"
				good ""
				ask_questions
				confirm_params
				configure_package
				;;
			-h|--help)
				copyright
				good ""
				good "Please run"
				good ""
				good "$0 install"
				good "	for installation/upgrade mode"
				good ""
				good "$0 fix"
				good "	for configuration fixing mode"
				good ""
				;;
			*)
				accept_license
				good "Installation/Upgrade mode entered"
				good ""
				ask_questions
				confirm_params
				install_package
				configure_package
				run_postconf
				rejoice
				;;

		esac
	}

# here we go, let's have fun!

preconf

DATE=`date`

echo "INSTALLATION STARTED ON $DATE" >$INSTALLLOG
echo "INSTALLATION STARTED ON $DATE" >$ERRORLOG
getscriptparams $* 2>>$ERRORLOG | tee -a $INSTALLLOG
echo "INSTALLATION FINISHED ON $DATE" >>$INSTALLLOG
echo "INSTALLATION FINISHED ON $DATE" >>$ERRORLOG

exit 0


