#
# This file is not to be run directly
#

if [ `basename $0` = "functions.sh" ] ; then echo "This file should not be called this way. Source it from your scripts instead." ; exit ; fi

colorize()
	{
		if [ $TERM = "linux" ] || [ $TERM = "xterm" ] || [ $TERM = "xterm-color" ] || [ $TERM = "screen" ] && [ "$USE_COLORS" = "1" ]
		then
			GOOD=$'\e[32;01m'
			WARN=$'\e[33;01m'
			BAD=$'\e[31;01m'
			NORMAL=$'\e[0m'
			HILITE=$'\e[37;01m'
			BRACKET=$'\e[34;01m'
		else
			GOOD=$'\e[0;32m'
			BAD=$'\e[0;31m'
			NORMAL=$'\e[0m'
			WARN=${NORMAL}
			HILITE=${NORMAL}
			BRACKET=${NORMAL}
		fi
	}
	
doinit()
	{
		source $1
		[[ $InstallDir != "" ]] && MERAKDIR="$InstallDir"
		[[ $User != "" ]] && MERAKUSER=$User
		[[ $Group != "" ]] && MERAKGROUP=$Group
		[[ $Chroot != "" ]] && CHROOT=$Chroot
		[[ $Version != "" ]] && VERSION=$Version		
		[[ $Capabilities != "" ]] && MERAKCAP="$Capabilities"
		REALMERAKDIR="${MERAKDIR}"
		ERRORLOG="${MERAKDIR}/logs/wizard/merak-wizard-error.log"
		WIZARDLOG="${MERAKDIR}/logs/wizard/merak-wizard.log"
		SCRIPTDIR="${MERAKDIR}/scripts"
		TOOL="${MERAKDIR}/tool"
		MERAKLIBPATH="lib/:$MERAKDIR/lib:$MERAKDIR/php/lib:$MERAKDIR/avast"
		MERAKCONF="${MERAKDIR}/config/merak.conf"
		MERAKUID=`id -u $MERAKUSER`
		MERAKGID=`grep "^$MERAKGROUP:" /etc/group |cut -d':' -f3`
		INITIALIZED="1"
	}

preconf()
	{
		pushd $REALPWD >/dev/null
		SEARCHCONF="config/merak.conf ../config/merak.conf ../../config/merak.conf"
		for TEMPCONF in $SEARCHCONF
			do
				if [ -f $TEMPCONF ]
					then
						MERAKCONF="$TEMPCONF"
						doinit $MERAKCONF
						break 1
				fi
		done
		
		popd >/dev/null
		
		colorize
	}
	
regenmerakconf()
	{
	                                echo "## Merak Mail Server configuration file
## this file was generated by regenmerakconf() on $DATE
## InstallDir=/path/to/merak
## Merak installation prefix, absolute path, no useful default

InstallDir=$MERAKDIR

## Version=number
## Merak Mail Server version, for reference only

Version=$VERSION

## User=username
## User to run services as, default=root

User=$MERAKUSER

## Group=group
## Group to run services as, default=root

Group=$MERAKGROUP

## Capabilities
## capabilities that are set at startup, see
## man cap_from_text
## for more info
## ADVANCED USERS ONLY!

Capabilities=$MERAKCAP

## Chroot=[0/1]
## Use chroot, default=0, currently disabled

## Chroot=$CHROOT" >$MERAKCONF
	}
	
toolquery()
	{
	pushd $MERAKDIR >/dev/null
	RESULT=`$TOOL -quiet "$1" "$2" "$3" "$4" "$5" "$6" "$7" "$8" "$9" "$10"`
	EXITCODE=$?
	popd >/dev/null
	return $EXITCODE
	}

tool()
	{
	pushd $MERAKDIR >/dev/null
	$TOOL -quiet "$1" "$2" "$3" "$4" "$5" "$6" "$7" "$8" "$9" "$10"
	EXITCODE=$?
	popd >/dev/null
	return $EXITCODE
	}


good()
	{
		echo "${GOOD}**	 ${NORMAL}$*"
	}


bad()
	{
		echo "${BAD}**	${NORMAL}$*"
	}

verybad()
	{
		echo "${BAD}************${HILITE}************${BAD}************${NORMAL}"
		echo "${HILITE}$*"
		echo "${BAD}************${HILITE}************${BAD}************${NORMAL}"
		exit 1
	}

warn()
	{
		echo "${WARN}**	 ${NORMAL}$*"
	}

hilite()
	{
		echo "${GOOD}**	${HILITE}$*${NORMAL}"
	}

getparam()
	{
		echo -n "${GOOD}**	${NORMAL}$1 "
		read PARAM
	}


getconfirm()
	{
		echo -n "${GOOD}**	${NORMAL}$* [${HILITE}Y${NORMAL}/n] "
		read PARAM
		if [ "$PARAM" = "" ]
			then
				PARAM="YES"
				return 0
		fi
		case $PARAM in
			y|Y|YES|Yes|yes)
				return 0
				;;
			*)
				return 1
				;;
		esac	
	}

delgetconfirm()
	{
		echo -n "${GOOD}**	${BAD}$* ${NORMAL}[y/${HILITE}N${NORMAL}] "
		read PARAM
		if [ "$PARAM" = "" ]
			then
				PARAM="NO"
				return 1
		fi
		
		case $PARAM in
			y|Y|YES|Yes|yes)
				return 0
				;;
			*)
				return 1
				;;
		esac	
	}
	
getsparam()
	{
		DIFFER="1"
		while [[ $DIFFER == "1" ]];
			do
			        echo -n "${GOOD}**	${NORMAL}$1 "
				read -s TOKEN
				good ""
				echo -n "${GOOD}**	${NORMAL}Once again to confirm: "
				read -s TOKEN_CONFIRM
				good ""

			        if [[ $TOKEN == $TOKEN_CONFIRM ]]
					then
						SPARAM="$TOKEN"
						DIFFER="0"
					else
						bad ""
						bad "Passwords don't match."
						bad "Try again"
						bad ""
				fi
		done
	}


menu()
	{
		while true
			do
				display_menu "$1" "$2" "$3"
				[[ "$PARAM" = "0" ]] && return
				[[ "$PARAM" = "Q" ]] && exit 0
				[[ "$PARAM" = "q" ]] && exit 0
				IFS="
"
				for LINE in `cat $3 | grep "^$2:"`
					do
						unset IFS
						LINECHOICE=`echo $LINE |cut -d':' -f2`
						NAME=`echo $LINE | cut -d':' -f3`
						TYPE=`echo $LINE | cut -d':' -f4`
						if [ "$LINECHOICE" = "$PARAM" ]
							then
								unset PARAM
								case $TYPE in
									script|SCRIPT)
										good ""
										SCRIPT=`echo $LINE | cut -d':' -f5`
										pushd `pwd` >/dev/null
										cd $MERAKDIR
										${SCRIPTDIR}/${SCRIPT} || bad "Error running script!"
										popd >/dev/null
										;;
								
									menu|MENU)
										MENUROOT=`echo $LINE | cut -d':' -f5`
										if [ "$MENUROOT" != "QUIT" ]
											then
												clear
												menu "$NAME" "$MENUROOT" "$3"
												clear

										fi
										;;
									*)
										MATCHED="0"
										;;
								esac
					fi
				done
		done
	}

display_menuitem()
	{
		CHOICE="$1"
		NAME="$2"
		echo "${GOOD}**	[${HILITE}${CHOICE}${GOOD}]${HILITE} ${NAME}${NORMAL}"
	}


display_menu()
	{
		good ""
		hilite "$1"
		good "You have the following options:"
		good ""
		cat $3 | grep "^$2:" | \
		while read LINE
		do
				NAME=`echo $LINE | cut -d':' -f2`
				LINECHOICE=`echo $LINE |cut -d':' -f3`
				display_menuitem "$NAME" "$LINECHOICE"
		done
		good ""
		display_menuitem "0" "Return"
		display_menuitem "Q" "Exit"
		good ""
		getparam "Enter your choice:"
	}

dochroot()
	{
		[[ ${CHROOT} == "1" ]] || return 0
		LD_LIBRARY_PATH="/lib:/avast"
		REALMERAKDIR="${MERAKDIR}"
		MERAKDIR="/"
		SCRIPTDIR="/scripts"
		CHROOTCMD="chroot ${REALMERAKDIR}"
		NOWCHROOTED="1"
		export LD_LIBRARY_PATH MERAKDIR REALMERAKDIR CHROOTCMD SCRIPTDIR NOWCHROOTED
	}

dounchroot()
	{
		[[ ${CHROOT} == "1" ]] || return 0
		MERAKDIR="${REALMERAKDIR}"
		unset CHROOTCMD
		export MERAKDIR
		preconf
	}
	
mysqlcall()
        {

	    local USERSTRING="-u${DBUSER}"
            [[ "$DBUSER" = "" ]] && local USERSTRING=""

	    local PASSSTRING="-p${DBUSERPW}"
            [[ "$DBUSERPW" = "" ]] && local PASSSTRING=""

	    local HOSTSTRING="-h${DBHOST}"
            [[ "$DBHOST" = "" ]] && local HOSTSTRING=""
	    
	    ( echo "${DBHOST}" | grep -q "/" ) && local HOSTSTRING="-S${DBHOST}"
	    
            echo "$SQLQUERY" | mysql $USERSTRING $PASSSTRING $HOSTSTRING $DBNAME

        }

mysqldumpin()
        {
            USER="$1"
            PASSWORD="$2"
            HOST="$3"
	    DB="$4"
            DUMPFILE="$5"
	    
	    PASSSTRING="-p${PASSWORD}"
            [[ "$PASSWORD" = "" ]] && PASSSTRING=""
            mysql -u"$USER" "$PASSSTRING" -h"$HOST" "$DB" <"${DUMPFILE}"
        }

isdaemonalive()
	{
		DAEMON="$1"
		PID=`cat "$MERAKDIR/var/$DAEMON.pid" 2>/dev/null`
                kill -0 $PID 2>/dev/null
	}
rootordie()
	{
		if [ `id -u` != "0" ]
			then
				warn "Must be run as root"
				exit 1
			else
				return 0
		fi
	}

preconf
